# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""The callable class of the validation in algorithm.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

from time import time
from numpy import zeros
cimport cython
from libc.stdlib cimport rand, srand, RAND_MAX

srand(int(time()))


cdef inline double rand_v(double lower = 0., double upper = 1.) nogil:
    """Random real value between [lower, upper]."""
    return lower + <double>rand() / RAND_MAX * (upper - lower)


cdef inline int rand_i(int upper) nogil:
    """Random integer between [0, upper]."""
    return rand() % upper


@cython.final
@cython.freelist(100)
cdef class Chromosome:

    """Data structure class."""

    def __cinit__(self, n: cython.int):
        self.n = n if n > 0 else 2
        self.f = 0.
        self.v = zeros(n)

    cdef void assign(self, Chromosome other):
        if other is self:
            return
        self.n = other.n
        self.f = other.f
        self.v = other.v.copy()


cdef class Verification:

    """Verification function class base."""

    cdef ndarray[double, ndim=1] get_upper(self):
        """Return upper bound."""
        raise NotImplementedError

    cdef ndarray[double, ndim=1] get_lower(self):
        """Return lower bound."""
        raise NotImplementedError

    cdef double fitness(self, ndarray[double, ndim=1] v):
        """Calculate the fitness.

        Usage:
        f = MyVerification()
        fitness = f(chromosome.v)
        """
        raise NotImplementedError

    cpdef object result(self, ndarray[double, ndim=1] v):
        """Show the result."""
        raise NotImplementedError



# -*- coding: utf-8 -*-

"""Lark parser to parse the expression."""

__author__ = "Yuan Chang"
__copyright__ = "Copyright (C) 2016-2019"
__license__ = "AGPL"
__email__ = "pyslvs@gmail.com"

from typing import (
    Tuple,
    List,
    Dict,
    Iterator,
    Union,
    Optional,
)
from lark import Lark, Transformer, LexError
from lark.lexer import Token
try:
    from .expression import (
        get_vlinks,
        VJoint,
        VPoint,
        VLink,
    )
    from .graph import Graph
except ImportError:
    from expression import (
        get_vlinks,
        VJoint,
        VPoint,
        VLink,
    )
    from graph import Graph

# Color dictionary.
_color_list: Dict[str, Tuple[int, int, int]] = {
    'Red': (172, 68, 68),
    'Green': (110, 190, 30),
    'Blue': (68, 120, 172),
    'Cyan': (0, 255, 255),
    'Magenta': (255, 0, 255),
    'Brick-Red': (255, 130, 130),
    'Yellow': (255, 255, 0),
    'Gray': (160, 160, 160),
    'Orange': (225, 165, 0),
    'Pink': (225, 192, 230),
    'Black': (0, 0, 0),
    'White': (255, 255, 255),
    'Dark-Red': (128, 0, 0),
    'Dark-Green': (0, 128, 0),
    'Dark-Blue': (0, 0, 128),
    'Dark-Cyan': (128, 0, 128),
    'Dark-Magenta': (255, 0, 255),
    'Dark-Yellow': (128, 128, 0),
    'Dark-Gray': (128, 128, 128),
    'Dark-Orange': (225, 140, 0),
    'Dark-Pink': (225, 20, 147),
}

color_names = tuple(sorted(_color_list.keys()))


def color_rgb(name: str) -> Tuple[int, int, int]:
    """Get color by name.

    + Invalid color
    + Color key
    + RGB string "(r, g, b)".
    """
    if name in _color_list:
        return _color_list[name]
    else:
        try:
            # Input RGB as a "(255, 255, 255)" string.
            color_text: Tuple[int, int, int] = tuple(int(i) for i in (
                name.replace('(', '')
                .replace(')', '')
                .replace(" ", '')
                .split(',', maxsplit=3)
            ))
        except ValueError:
            return 0, 0, 0
        else:
            return color_text


_GRAMMAR = Lark(r"""
    // Number
    DIGIT: "0".."9"
    INT: DIGIT+
    SIGNED_INT: ["+" | "-"] INT
    DECIMAL: INT "." INT? | "." INT
    _EXP: ("e" | "E") SIGNED_INT
    FLOAT: INT _EXP | DECIMAL _EXP?
    NUMBER: FLOAT | INT
    SIGNED_NUMBER: ["+" | "-"] NUMBER

    // Letters
    LCASE_LETTER: "a".."z"
    UCASE_LETTER: "A".."Z"
    LETTER: UCASE_LETTER | LCASE_LETTER | "_"
    CNAME: LETTER (LETTER | DIGIT)*

    // White space and new line
    WS: /\s+/
    CR: /\r/
    LF: /\n/
    NEWLINE: (CR? LF)+
    %ignore WS
    %ignore NEWLINE

    // Comment
    LINE_COMMENT: /#[^\n]*/
    MULTILINE_COMMENT: /#\[[\s\S]*#\][^\n]*/
    %ignore LINE_COMMENT
    %ignore MULTILINE_COMMENT

    // Custom data type
    JOINT_TYPE: "RP" | "R" | "P"
    COLOR: """ + "|".join(f'"{color}"' for color in color_names) + r"""
    type: JOINT_TYPE
    name: CNAME
    number: SIGNED_NUMBER
    color_value: INT

    // Main grammar
    joint: "J[" type ["," angle] ["," color] "," point "," link "]"
    link: "L[" [name ("," name)* ","?] "]"
    point: "P[" number "," number "]"
    angle: "A[" number "]"
    color: "color[" (("(" color_value ("," color_value) ~ 2 ")") | COLOR) "]"
    mechanism: "M[" [joint ("," joint)* ","?] "]"
    ?start: mechanism
""", parser='lalr')


class _ParamsTrans(Transformer):

    """Transformer will parse into a list of VPoint data."""

    @staticmethod
    def type(n: List[Token]) -> str:
        return str(n[0])

    name = type

    @staticmethod
    def color(n: List[Token]) -> str:
        return str(n[0]) if len(n) == 1 else str(tuple(n))

    @staticmethod
    def color_value(n: List[Token]) -> int:
        return int(n[0])

    @staticmethod
    def number(n: List[Token]) -> float:
        return float(n[0])

    @staticmethod
    def point(c: List[Token]) -> Tuple[Token]:
        return tuple(c)

    angle = number

    @staticmethod
    def link(a: List[Token]) -> Tuple[Token]:
        return tuple(a)

    @staticmethod
    def joint(args: List[Token]) -> List[Union[str, int, float]]:
        """Sort the argument list.

        [0]: type
        ([1]: angle)
        ([2]: color)
        [-2]: point (coordinate)
        [-1]: link
        """
        type_str = args[0]
        x, y = args[-2]
        links = ','.join(args[-1])
        if type_str == 'R':
            if len(args) == 3:
                return [links, 'R', 'Green', x, y]
            elif len(args) == 4:
                return [links, 'R', args[-3], x, y]
        else:
            type_angle = f'{args[0]}:{args[1]}'
            if len(args) == 4:
                return [links, type_angle, 'Green', x, y]
            elif len(args) == 5:
                return [links, type_angle, args[-3], x, y]

        raise LexError(f"invalid options: {args}")

    @staticmethod
    def mechanism(joints: List[Token]) -> List[Token]:
        return joints


class _PositionTrans(_ParamsTrans):

    """Transformer will parse into a list of position data."""

    @staticmethod
    def joint(args: List[Token]) -> Tuple[float, float]:
        x, y = args[-2]
        return x, y


class _VPointsTrans(_ParamsTrans):

    """Using same grammar return as VPoints."""

    @staticmethod
    def type(n: List[Token]) -> VJoint:
        """Return as int type."""
        type_str = str(n[0])
        if type_str == 'R':
            return VJoint.R
        elif type_str == 'P':
            return VJoint.P
        elif type_str == 'RP':
            return VJoint.RP

    @staticmethod
    def joint(args: List[Token]) -> VPoint:
        """Same as parent."""
        type_int = args[0]
        x, y = args[-2]
        links: Tuple[str, ...] = args[-1]
        if type_int == VJoint.R:
            if len(args) == 3:
                return VPoint.r_joint(links, x, y)
            elif len(args) == 4:
                return VPoint(links, VJoint.R, 0., args[-3], x, y, color_rgb)
        else:
            if len(args) == 4:
                return VPoint.slider_joint(links, type_int, args[1], x, y)
            elif len(args) == 5:
                return VPoint(links, type_int, args[1], args[-3], x, y, color_rgb)

        raise LexError(f"invalid options: {args}")


_params_translator = _ParamsTrans()
_pos_translator = _PositionTrans()
_vpoint_translator = _VPointsTrans()


def parse_params(expr: str) -> List[List[Union[str, float]]]:
    """Using to parse the expression and return arguments."""
    return _params_translator.transform(_GRAMMAR.parse(expr))


def parse_pos(expr: str) -> List[Tuple[float, float]]:
    """Using to parse the expression and return arguments."""
    return _pos_translator.transform(_GRAMMAR.parse(expr))


def parse_vpoints(expr: str) -> List[VPoint]:
    """Parse as VPoints."""
    return _vpoint_translator.transform(_GRAMMAR.parse(expr))


def parse_vlinks(expr: str) -> List[VLink]:
    """Parse as VLinks."""
    return get_vlinks(parse_vpoints(expr))


def edges_view(graph: Graph) -> Iterator[Tuple[int, Tuple[int, int]]]:
    """This generator can keep the numbering be consistent."""
    yield from enumerate(sorted(tuple(sorted(e)) for e in graph.edges))


def graph2vpoints(
    graph: Graph,
    pos: Dict[int, Tuple[float, float]],
    cus: Optional[Dict[int, int]] = None,
    same: Optional[Dict[int, int]] = None,
    grounded: Optional[int] = None
) -> List[VPoint]:
    """Change NetworkX graph into VPoints.

    cus: custom nodes (not joint)
        {node_name: link_number}
    same: multiple joints
        {n1: n2, n3: n2} => (n1 as n2) and (n3 as n2)
    """
    if cus is None:
        cus: Dict[int, int] = {}
    if same is None:
        same: Dict[int, int] = {}

    same_r = {}
    for k, v in same.items():
        if v in same_r:
            same_r[v].append(k)
        else:
            same_r[v] = [k]
    tmp_list = []
    ev = dict(edges_view(graph))
    for i, edge in ev.items():
        if i in same:
            # Do not connect to anyone!
            continue
        edge = set(edge)
        if i in same_r:
            for j in same_r[i]:
                edge.update(set(ev[j]))
        x, y = pos[i]
        links = [(f"L{link}" if link != grounded else 'ground') for link in edge]
        tmp_list.append(VPoint.r_joint(links, x, y))
    for name in sorted(cus):
        link = f"L{cus[name]}" if cus[name] != grounded else 'ground'
        x, y = pos[name]
        tmp_list.append(VPoint.r_joint((link,), x, y))
    return tmp_list


try:
    from pygments.lexer import RegexLexer
    from pygments.token import (
        Comment,
        Keyword,
        Name,
        Number,
    )
except ImportError:
    HAS_PYGMENTS = False
else:
    HAS_PYGMENTS = True

    class PMKSLexer(RegexLexer):

        """Mechanism expression highlighter by Pygments."""

        name = 'Mechanism Expression'

        tokens = {
            'root': [
                (r'#[^\n]*', Comment.Single),
                (r'#\[[\s\S]*#\][^\n]*', Comment.Multiple),
                (r'M|J|L|P|A|color', Name.Function),
                ('|'.join(f"{color}" for color in reversed(color_names)), Name.Variable),
                (r'RP|R|P', Keyword.Constant),
                (r'(\d+\.\d*|\.\d+)([eE][+-]?\d+)?', Number.Float),
            ]
        }



# -*- coding: utf-8 -*-

"""Kernel of Metaheuristic Random Algorithm."""

from .verify import Verification
from .rga import Genetic
from .firefly import Firefly
from .de import Differential

__all__ = [
    'Verification',
    'Genetic',
    'Firefly',
    'Differential',
]



# -*- coding: utf-8 -*-

from typing import (
    Tuple,
    List,
    Sequence,
    Dict,
    Union,
    Optional,
)
from .triangulation import ExpressionStack
from .expression import VPoint, Coordinate

TuplePoint = Tuple[float, float]


def plap(
    c1: Coordinate,
    d0: float,
    a0: float,
    c2: Optional[Coordinate] = None,
    inverse: bool = False
) -> Coordinate:
    """Point on circle by angle."""
    ...


def pllp(
    c1: Coordinate,
    d0: float,
    d1: float,
    c2: Coordinate,
    inverse: bool = False
) -> Coordinate:
    """Two intersection points of two circles."""
    ...


def plpp(
    c1: Coordinate,
    d0: float,
    c2: Coordinate,
    c3: Coordinate,
    inverse: bool = False
) -> Coordinate:
    """Two intersection points of a line and a circle."""
    ...


def pxy(c1: Coordinate, x: float, y: float) -> Coordinate:
    """Using relative cartesian coordinate to get solution."""
    ...


def vpoint_dof(vpoints: Sequence[VPoint]) -> int:
    """Degree of freedoms calculate from PMKS expressions."""
    ...


def expr_parser(exprs: Sequence[Tuple[str, ...]], data_dict: Dict[str, float]):
    """Update data."""
    ...


def expr_solving(
    exprs: ExpressionStack,
    mapping: Dict[Union[int, Tuple[int, int]], Union[str, float]],
    vpoints: Sequence[VPoint],
    angles: Sequence[float] = None
) -> List[Union[TuplePoint, Tuple[TuplePoint, TuplePoint]]]:
    """Solving function."""
    ...


def data_collecting(
    exprs: ExpressionStack,
    mapping: Dict[int, str],
    vpoints_: Sequence[VPoint],
) -> Tuple[Dict[str, Union[Coordinate, float]], int]:
    """Data collecting process."""
    ...



# -*- coding: utf-8 -*-

"""Compile the Cython libraries of Pyslvs."""

from os import listdir
from distutils.core import setup, Extension
from platform import system
from Cython.Distutils import build_ext
import numpy

np_include = numpy.get_include()


sources = []
for place in ["."]:
    for source in listdir(place):
        if source.endswith('.pyx'):
            sources.append(place + source)

macros = [
    ('_hypot', 'hypot'),
]

compile_args = [
    '-O3',
]

if system() == 'Windows':
    # Avoid compile error with CYTHON_USE_PYLONG_INTERNALS.
    # https://github.com/cython/cython/issues/2670#issuecomment-432212671
    macros.append(('MS_WIN64', None))
    # Disable format warning.
    compile_args.append('-Wno-format')
    # Disable NumPy warning.
    macros.append(('NPY_NO_DEPRECATED_API', 'NPY_1_7_API_VERSION'))

# Original src
ext_modules = []
for source in sources:
    ext_modules.append(Extension(
        # Base name
        source.split('/')[-1].split('.')[0],
        # path + file name
        sources=[source],
        language="c",
        include_dirs=[np_include],
        define_macros=macros,
        extra_compile_args=compile_args,
    ))

setup(ext_modules=ext_modules, cmdclass={'build_ext': build_ext})



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""Real-coded Genetic Algorithm.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

from time import time
from libc.math cimport pow, HUGE_VAL
cimport cython
from numpy cimport ndarray
from .verify cimport (
    Limit,
    MAX_GEN,
    MIN_FIT,
    MAX_TIME,
    rand_v,
    rand_i,
    Chromosome,
    Verification,
)


@cython.final
cdef class Genetic:

    """Algorithm class."""

    cdef Limit option
    cdef int nParm, nPop, max_gen, max_time, gen, rpt
    cdef double pCross, pMute, pWin, bDelta, min_fit, time_start
    cdef Verification func
    cdef object progress_fun, interrupt_fun
    cdef ndarray chromosome, new_chromosome, ub, lb
    cdef Chromosome last_best
    cdef list fitness_time

    def __cinit__(
        self,
        func: Verification,
        settings: dict,
        progress_fun: object = None,
        interrupt_fun: object = None
    ):
        """
        settings = {
            'nPop',
            'pCross',
            'pMute',
            'pWin',
            'bDelta',
            'max_gen' or 'min_fit' or 'max_time',
            'report'
        }
        """
        self.func = func

        self.nPop = settings.get('nPop', 500)
        self.pCross = settings.get('pCross', 0.95)
        self.pMute = settings.get('pMute', 0.05)
        self.pWin = settings.get('pWin', 0.95)
        self.bDelta = settings.get('bDelta', 5.)
        self.max_gen = 0
        self.min_fit = 0
        self.max_time = 0
        if 'max_gen' in settings:
            self.option = MAX_GEN
            self.max_gen = settings['max_gen']
        elif 'min_fit' in settings:
            self.option = MIN_FIT
            self.min_fit = settings['min_fit']
        elif 'max_time' in settings:
            self.option = MAX_TIME
            self.max_time = settings['max_time']
        else:
            raise ValueError("please give 'max_gen', 'min_fit' or 'max_time' limit")
        self.rpt = settings.get('report', 0)
        if self.rpt <= 0:
            self.rpt = 10
        self.progress_fun = progress_fun
        self.interrupt_fun = interrupt_fun

        # low bound
        self.lb = self.func.get_lower()
        # up bound
        self.ub = self.func.get_upper()
        if len(self.lb) != len(self.ub):
            raise ValueError("length of upper and lower bounds must be equal")
        self.nParm = len(self.lb)

        self.chromosome = ndarray(self.nPop, dtype=object)
        self.new_chromosome = ndarray(self.nPop, dtype=object)
        cdef int i
        for i in range(self.nPop):
            self.chromosome[i] = Chromosome.__new__(Chromosome, self.nParm)
        for i in range(self.nPop):
            self.new_chromosome[i] = Chromosome.__new__(Chromosome, self.nParm)

        self.last_best = Chromosome.__new__(Chromosome, self.nParm)

        # generations
        self.gen = 0

        # setup benchmark
        self.time_start = -1
        self.fitness_time = []

    cdef inline double check(self, int i, double v):
        """If a variable is out of bound, replace it with a random value."""
        if v > self.ub[i] or v < self.lb[i]:
            return rand_v(self.lb[i], self.ub[i])
        return v

    cdef inline void initialize(self):
        cdef int i, j
        cdef Chromosome tmp
        for i in range(self.nPop):
            tmp = self.chromosome[i]
            for j in range(self.nParm):
                tmp.v[j] = rand_v(self.lb[j], self.ub[j])

        tmp = self.chromosome[0]
        tmp.f = self.func.fitness(tmp.v)
        self.last_best.assign(tmp)

    cdef inline void cross_over(self):
        cdef Chromosome c1 = Chromosome.__new__(Chromosome, self.nParm)
        cdef Chromosome c2 = Chromosome.__new__(Chromosome, self.nParm)
        cdef Chromosome c3 = Chromosome.__new__(Chromosome, self.nParm)

        cdef int i, s, j
        cdef Chromosome b1, b2
        for i in range(0, self.nPop - 1, 2):
            if not rand_v() < self.pCross:
                continue

            b1 = self.chromosome[i]
            b2 = self.chromosome[i + 1]
            for s in range(self.nParm):
                # first baby, half father half mother
                c1.v[s] = 0.5 * b1.v[s] + 0.5 * b2.v[s]
                # second baby, three quarters of father and quarter of mother
                c2.v[s] = self.check(s, 1.5 * b1.v[s] - 0.5 * b2.v[s])
                # third baby, quarter of father and three quarters of mother
                c3.v[s] = self.check(s, -0.5 * b1.v[s] + 1.5 * b2.v[s])
            # evaluate new baby
            c1.f = self.func.fitness(c1.v)
            c2.f = self.func.fitness(c2.v)
            c3.f = self.func.fitness(c3.v)
            # maybe use bubble sort? smaller -> larger
            if c1.f > c2.f:
                c1, c2 = c2, c1
            if c1.f > c3.f:
                c1, c3 = c3, c1
            if c2.f > c3.f:
                c2, c3 = c3, c2
            # replace first two baby to parent, another one will be
            b1.assign(c1)
            b2.assign(c2)

    cdef inline double delta(self, double y):
        cdef double r
        if self.max_gen > 0:
            r = <double>self.gen / self.max_gen
        else:
            r = 1
        return y * rand_v() * pow(1.0 - r, self.bDelta)

    cdef inline void fitness(self):
        cdef int i
        cdef Chromosome tmp
        for i in range(self.nPop):
            tmp = self.chromosome[i]
            tmp.f = self.func.fitness(tmp.v)

        cdef int index = 0
        cdef double f = HUGE_VAL

        for i, tmp in enumerate(self.chromosome):
            if tmp.f < f:
                index = i
                f = tmp.f
        if f < self.last_best.f:
            self.last_best.assign(self.chromosome[index])

    cdef inline void mutate(self):
        cdef int i, s
        cdef Chromosome tmp
        for i in range(self.nPop):
            if not rand_v() < self.pMute:
                continue
            s = rand_i(self.nParm)
            tmp = self.chromosome[i]
            if rand_v() < 0.5:
                tmp.v[s] += self.delta(self.ub[s] - tmp.v[s])
            else:
                tmp.v[s] -= self.delta(tmp.v[s] - self.lb[s])

    cdef inline void report(self):
        self.fitness_time.append((self.gen, self.last_best.f, time() - self.time_start))

    cdef inline void select(self):
        """
        roulette wheel selection
        """
        cdef int i, j, k
        cdef Chromosome baby, b1, b2
        for i in range(self.nPop):
            j = rand_i(self.nPop)
            k = rand_i(self.nPop)
            b1 = self.chromosome[j]
            b2 = self.chromosome[k]
            baby = self.new_chromosome[i]
            if b1.f > b2.f and rand_v() < self.pWin:
                baby.assign(b2)
            else:
                baby.assign(b1)
        # in this stage, new_chromosome is select finish
        # now replace origin chromosome
        for i in range(self.nPop):
            baby = self.chromosome[i]
            baby.assign(self.new_chromosome[i])
        # select random one chromosome to be best chromosome, make best chromosome still exist
        baby = self.chromosome[rand_i(self.nPop)]
        baby.assign(self.last_best)

    cdef inline void generation_process(self):
        self.gen += 1

        self.select()
        self.cross_over()
        self.mutate()
        self.fitness()
        if self.gen % self.rpt == 0:
            self.report()

    cpdef tuple run(self):
        """Init and run GA for max_gen times."""
        self.time_start = time()
        self.initialize()
        self.fitness()
        self.report()

        while True:
            self.generation_process()

            if self.option == MAX_GEN:
                if self.gen >= self.max_gen > 0:
                    break
            elif self.option == MIN_FIT:
                if self.last_best.f <= self.min_fit:
                    break
            elif self.option == MAX_TIME:
                if time() - self.time_start >= self.max_time > 0:
                    break

            # progress
            if self.progress_fun is not None:
                self.progress_fun(self.gen, f"{self.last_best.f:.04f}")

            # interrupt
            if self.interrupt_fun is not None and self.interrupt_fun():
                break

        self.report()
        return self.func.result(self.last_best.v), self.fitness_time



# -*- coding: utf-8 -*-

from typing import (
    List,
    Tuple,
    Sequence,
    Callable,
    Optional,
)


def link_synthesis(
    nl: int,
    nj: int,
    stop_func: Optional[Callable[[], bool]] = None
) -> List[Tuple[int, ...]]:
    """Generate the link assortment."""
    ...


def contracted_link_synthesis(
    link_num_list: Sequence[int],
    stop_func: Optional[Callable[[], bool]] = None
) -> List[Tuple[int, ...]]:
    """Generate the contracted link assortment."""
    ...



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""Differential Evolution.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

from time import time
cimport cython
from libc.math cimport HUGE_VAL
from numpy cimport ndarray
from .verify cimport (
    Limit,
    MAX_GEN,
    MIN_FIT,
    MAX_TIME,
    rand_v,
    rand_i,
    Chromosome,
    Verification,
)

ctypedef void (*Eq)(Differential, int, Chromosome)


@cython.final
cdef class Differential:

    """Algorithm class."""

    cdef Limit option
    cdef int strategy, D, NP, max_gen, max_time, rpt, gen, r1, r2, r3, r4, r5
    cdef double F, CR, min_fit, time_start
    cdef ndarray lb, ub, pool
    cdef Verification func
    cdef object progress_fun, interrupt_fun
    cdef Chromosome last_best
    cdef list fitness_time

    def __cinit__(
        self,
        func: Verification,
        settings: dict,
        progress_fun: object = None,
        interrupt_fun: object = None
    ):
        """
        settings = {
            'strategy',
            'NP',
            'F',
            'CR',
            'max_gen', 'min_fit' or 'max_time',
            'report'
        }
        """
        # object function, or environment
        self.func = func

        # strategy 0~9, choice what strategy to generate new member in temporary
        self.strategy = settings.get('strategy', 1)
        # population size
        # To start off NP = 10*D is a reasonable choice. Increase NP if misconvergence
        self.NP = settings.get('NP', 400)
        # weight factor
        # F is usually between 0.5 and 1 (in rare cases > 1)
        self.F = settings.get('F', 0.6)
        # crossover possible
        # CR in [0,1]
        self.CR = settings.get('CR', 0.9)
        # low bound
        self.lb = self.func.get_lower()
        # up bound
        self.ub = self.func.get_upper()
        if len(self.lb) != len(self.ub):
            raise ValueError("length of upper and lower bounds must be equal")
        # dimension of question
        self.D = len(self.lb)
        # Algorithm will stop when the limitation has happened.
        self.max_gen = 0
        self.min_fit = 0
        self.max_time = 0
        if 'max_gen' in settings:
            self.option = MAX_GEN
            self.max_gen = settings['max_gen']
        elif 'min_fit' in settings:
            self.option = MIN_FIT
            self.min_fit = settings['min_fit']
        elif 'max_time' in settings:
            self.option = MAX_TIME
            self.max_time = settings['max_time']
        else:
            raise ValueError("please give 'max_gen', 'min_fit' or 'max_time' limit")
        # Report function
        self.rpt = settings.get('report', 0)
        if self.rpt <= 0:
            self.rpt = 10
        self.progress_fun = progress_fun
        self.interrupt_fun = interrupt_fun

        # check parameter is set properly
        self.check_parameter()

        # generation pool, depend on population size
        self.pool = ndarray(self.NP, dtype=object)
        cdef int i
        for i in range(self.NP):
            self.pool[i] = Chromosome.__new__(Chromosome, self.D)

        # last generation best member
        self.last_best = Chromosome.__new__(Chromosome, self.D)

        # the generation count
        self.gen = 0

        # the vector
        self.r1 = self.r2 = self.r3 = self.r4 = self.r5 = 0

        # setup benchmark
        self.time_start = -1
        self.fitness_time = []

    cdef inline void check_parameter(self):
        """Check parameter is set properly."""
        if self.D <= 0:
            raise ValueError('D should be integer and larger than 0')
        if self.NP <= 0:
            raise ValueError('NP should be integer and larger than 0')
        if not (0 <= self.CR <= 1):
            raise ValueError('CR should be [0,1]')
        if self.strategy not in range(10):
            raise ValueError('strategy should be [0,9]')

    cdef inline void initialize(self):
        """Initial population."""
        cdef int i, j
        cdef Chromosome tmp
        for i in range(self.NP):
            tmp = self.pool[i]
            for j in range(self.D):
                tmp.v[j] = rand_v(self.lb[j], self.ub[j])
            tmp.f = self.func.fitness(tmp.v)

    cdef inline Chromosome find_best(self):
        """Find member that have minimum fitness value from pool."""
        cdef int index = 0
        cdef double f = HUGE_VAL

        cdef int i
        cdef Chromosome tmp
        for i, tmp in enumerate(self.pool):
            if tmp.f < f:
                index = i
                f = tmp.f
        return self.pool[index]

    cdef inline void generate_random_vector(self, int i):
        """Generate new vector."""
        self.r1 = self.r2 = self.r3 = self.r4 = self.r5 = i
        while self.r1 == i:
            self.r1 = rand_i(self.NP)
        while self.r2 in {i, self.r1}:
            self.r2 = rand_i(self.NP)

        if self.strategy in {1, 3, 6, 8}:
            return

        while self.r3 in {i, self.r1, self.r2}:
            self.r3 = rand_i(self.NP)

        if self.strategy in {2, 7}:
            return

        while self.r4 in {i, self.r1, self.r2, self.r3}:
            self.r4 = rand_i(self.NP)

        if self.strategy in {4, 9}:
            return

        while self.r5 in {i, self.r1, self.r2, self.r3, self.r4}:
            self.r5 = rand_i(self.NP)

    cdef inline void type1(self, Chromosome tmp, Eq func):
        cdef int n = rand_i(self.D)
        cdef int l_v = 0
        while True:
            func(self, n, tmp)
            n = (n + 1) % self.D
            l_v += 1
            if not (rand_v() < self.CR and l_v < self.D):
                break

    cdef inline void type2(self, Chromosome tmp, Eq func):
        cdef int n = rand_i(self.D)
        cdef int l_v
        for l_v in range(self.D):
            if rand_v() < self.CR or l_v == self.D - 1:
                func(self, n, tmp)
            n = (n + 1) % self.D

    cdef void eq1(self, int n, Chromosome tmp):
        cdef Chromosome c1 = self.pool[self.r1]
        cdef Chromosome c2 = self.pool[self.r2]
        tmp.v[n] = self.last_best.v[n] + self.F * (c1.v[n] - c2.v[n])

    cdef void eq2(self, int n, Chromosome tmp):
        cdef Chromosome c1 = self.pool[self.r1]
        cdef Chromosome c2 = self.pool[self.r2]
        cdef Chromosome c3 = self.pool[self.r3]
        tmp.v[n] = c1.v[n] + self.F * (c2.v[n] - c3.v[n])

    cdef void eq3(self, int n, Chromosome tmp):
        cdef Chromosome c1 = self.pool[self.r1]
        cdef Chromosome c2 = self.pool[self.r2]
        tmp.v[n] = tmp.v[n] + self.F * (self.last_best.v[n] - tmp.v[n]) + self.F * (c1.v[n] - c2.v[n])

    cdef void eq4(self, int n, Chromosome tmp):
        cdef Chromosome c1 = self.pool[self.r1]
        cdef Chromosome c2 = self.pool[self.r2]
        cdef Chromosome c3 = self.pool[self.r3]
        cdef Chromosome c4 = self.pool[self.r4]
        tmp.v[n] = self.last_best.v[n] + (c1.v[n] + c2.v[n] - c3.v[n] - c4.v[n]) * self.F

    cdef void eq5(self, int n, Chromosome tmp):
        cdef Chromosome c1 = self.pool[self.r1]
        cdef Chromosome c2 = self.pool[self.r2]
        cdef Chromosome c3 = self.pool[self.r3]
        cdef Chromosome c4 = self.pool[self.r4]
        cdef Chromosome c5 = self.pool[self.r5]
        tmp.v[n] = c5.v[n] + (c1.v[n] + c2.v[n] - c3.v[n] - c4.v[n]) * self.F

    cdef inline Chromosome recombination(self, int i):
        """use new vector, recombination the new one member to tmp."""
        cdef Chromosome tmp = Chromosome.__new__(Chromosome, self.D)
        tmp.assign(self.pool[i])

        if self.strategy == 1:
            self.type1(tmp, Differential.eq1)
        elif self.strategy == 2:
            self.type1(tmp, Differential.eq2)
        elif self.strategy == 3:
            self.type1(tmp, Differential.eq3)
        elif self.strategy == 4:
            self.type1(tmp, Differential.eq4)
        elif self.strategy == 5:
            self.type1(tmp, Differential.eq5)
        elif self.strategy == 6:
            self.type2(tmp, Differential.eq1)
        elif self.strategy == 7:
            self.type2(tmp, Differential.eq2)
        elif self.strategy == 8:
            self.type2(tmp, Differential.eq3)
        elif self.strategy == 9:
            self.type2(tmp, Differential.eq4)
        elif self.strategy == 0:
            self.type2(tmp, Differential.eq5)
        return tmp

    cdef inline void report(self):
        self.fitness_time.append((self.gen, self.last_best.f, time() - self.time_start))

    cdef inline bint over_bound(self, Chromosome member):
        """check the member's chromosome that is out of bound?"""
        cdef int i
        for i in range(self.D):
            if member.v[i] > self.ub[i] or member.v[i] < self.lb[i]:
                return True
        return False

    cdef inline void generation_process(self):
        self.gen += 1

        cdef int i
        cdef Chromosome tmp, baby
        for i in range(self.NP):
            # generate new vector
            self.generate_random_vector(i)
            # use the vector recombine the member to temporary
            tmp = self.recombination(i)
            # check the one is out of bound?
            if self.over_bound(tmp):
                # if it is, then abandon it
                continue
            # is not out of bound, that mean it's qualify of environment
            # then evaluate the one
            tmp.f = self.func.fitness(tmp.v)
            # if temporary one is better than origin(fitness value is smaller)
            baby = self.pool[i]
            if tmp.f <= baby.f:
                # copy the temporary one to origin member
                baby.assign(tmp)
                # check the temporary one is better than the current_best
                if tmp.f < self.last_best.f:
                    # copy the temporary one to current_best
                    self.last_best.assign(tmp)

        if self.gen % self.rpt == 0:
            self.report()

    cpdef tuple run(self):
        """Run the algorithm."""
        self.time_start = time()
        # initialize the member's chromosome
        self.initialize()
        # find the best one (smallest fitness value)
        cdef Chromosome tmp = self.find_best()
        # copy to last_best
        self.last_best.assign(tmp)
        # report status
        self.report()

        # the evolution journey is begin ...
        while True:
            self.generation_process()

            if self.option == MAX_GEN:
                if self.gen >= self.max_gen > 0:
                    break
            elif self.option == MIN_FIT:
                if self.last_best.f <= self.min_fit:
                    break
            elif self.option == MAX_TIME:
                if time() - self.time_start >= self.max_time > 0:
                    break

            # progress
            if self.progress_fun is not None:
                self.progress_fun(self.gen, f"{self.last_best.f:.04f}")

            # interrupt
            if self.interrupt_fun is not None and self.interrupt_fun():
                break

        # the evolution journey is done, report the final status
        self.report()
        return self.func.result(self.last_best.v), self.fitness_time



# -*- coding: utf-8 -*-

from typing import (
    Tuple,
    Dict,
    Union,
    Any,
)
from numpy import ndarray
from .verify import Verification


class Planar(Verification):

    """This class is used to verified kinematics of the linkage mechanism."""

    def __init__(self, mech_params: Dict[str, Any]):
        """mech_params = {
            'Driver': {'pt': (x, y, r)},
            'Follower': {'pt': (x, y, r)},
            'Target': {'pt': [(x0, y0), (x1, y1), ...]},
            'constraints': [('pt', 'pt', 'pt', 'pt')],
            'Expression': str,
            'upper': ndarray[np_float32],
            'lower': ndarray[np_float32],
        }
        """
        ...

    def is_two_kernel(self) -> bool:
        ...

    def result(self, v: ndarray) -> str:
        """Return the last answer."""
        ...



# -*- coding: utf-8 -*-

from typing import (
    Tuple,
    List,
    Sequence,
    Dict,
    Optional,
)
from .expression import VPoint


class ExpressionStack:

    """The stack of Python wrapper."""

    def as_list(self) -> List[Tuple[str, ...]]:
        ...

    def __repr__(self) -> str:
        ...


def vpoints_configure(
    vpoints_: Sequence[VPoint],
    inputs: Sequence[Tuple[int, int]],
    status: Optional[Dict[int, bool]] = None
) -> ExpressionStack:
    """Auto configuration algorithm."""
    ...



# -*- coding: utf-8 -*-

from typing import Tuple, Dict
from graph import Graph


def external_loop_layout(
    graph: Graph,
    node_mode: bool,
    scale: float = 1.
) -> Dict[int, Tuple[float, float]]:
    """Layout position decided by outer loop (max cycle)."""
    ...



# -*- coding: utf-8 -*-

from typing import (
    Tuple,
    List,
    Dict,
    Callable,
    Optional,
    Any,
)
from .verify import Verification


class Differential:

    """Algorithm class."""

    def __init__(
        self,
        func: Verification,
        settings: Dict[str, Any],
        progress_fun: Optional[Callable[[int, str], None]] = None,
        interrupt_fun: Optional[Callable[[], bool]] = None
    ):
        """
        settings = {
            'nPop',
            'pCross',
            'pMute',
            'pWin',
            'bDelta',
            'max_gen' or 'min_fit' or 'max_time',
            'report'
        }
        """
        ...

    def run(self) -> Tuple[Any, List[Tuple[int, float, float]]]: ...



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""Graph class.

The algorithms references:
+ NetworkX

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

cimport cython
from libcpp.pair cimport pair as cpair
import sys
from typing import (
    Tuple,
    Iterable,
    Dict,
    Iterator,
)

ctypedef cpair[int, int] ipair


cpdef list link_assortment(Graph g):
    """Return link assortment of the graph."""
    cdef list assortment = [0]
    if not g.edges:
        return assortment

    cdef int d, n
    for n in g.degrees().values():
        if n < 2:
            continue
        d = n - 2
        while d >= len(assortment):
            assortment.append(0)
        assortment[d] += 1

    return assortment


cpdef list contracted_link_assortment(Graph g):
    """Return contracted link assortment of the graph."""
    if not g.edges:
        return [0]

    cdef list assortment = [0] * link_assortment(g)[0]

    cdef int d
    cdef tuple mcl
    cdef set counted = set()
    for mcl in _multi_contracted_links(g, False):
        d = len(mcl) - 1
        counted.update(mcl)
        assortment[d] += 1

    # For single contracted links.
    cdef int n
    for n, d in g.degrees().items():
        if d != 2:
            continue
        if n not in counted:
            assortment[0] += 1

    return assortment


cdef list _multi_contracted_links(Graph g, bint only_one):
    """Return a list of multiple contracted links."""
    cdef int n1, n2, index, neighbor
    cdef list c_links
    cdef list contracted_links = []
    cdef set counted = set()
    for n1 in g.nodes:
        # Only for binary link.
        if len(g.adj[n1]) != 2:
            continue

        # n1 is not collected yet.
        if n1 in counted:
            continue

        index = 0
        c_links = [n1]
        while index < len(c_links):
            for neighbor in g.adj[c_links[index]]:
                if len(g.adj[neighbor]) == 2:
                    if neighbor not in c_links:
                        c_links.append(neighbor)
            index += 1
        if len(c_links) > 1:
            if only_one:
                return c_links
            else:
                counted.update(c_links)
                contracted_links.append(tuple(c_links))

    if only_one:
        return []
    else:
        return contracted_links


cpdef list labeled_enumerate(Graph g):
    """Enumerate each node with labeled except isomorphism."""
    cdef list result = []

    cdef int n1, n2
    cdef Graph g1, g2
    for n1 in g.nodes:
        g1 = Graph.__new__(Graph, [e for e in g.edges if n1 not in e])
        for n2, g2 in result:
            if g1.is_isomorphic(g2):
                break
        else:
            result.append((n1, g1))

    return result


@cython.final
cdef class Graph:

    """NetworkX-like graph class."""

    def __cinit__(self, edges: Iterable[Tuple[int, int]]):
        """edges: [(l1, l2), ...]"""
        self.edges = tuple(edges)

        # nodes
        cdef int p1, p2
        cdef list nodes = []
        for p1, p2 in self.edges:
            if p1 not in nodes:
                nodes.append(p1)
            if p2 not in nodes:
                nodes.append(p2)
        self.nodes = tuple(nodes)

        # adj
        cdef int n
        self.adj = {n: self.neighbors(n) for n in self.nodes}

    cpdef void add_nodes(self, object new_nodes):
        """Add nodes from a iterable."""
        self.nodes = tuple(set(self.nodes) | set(new_nodes))

    cpdef void add_edge(self, int n1, int n2):
        """Add two nodes for an edge."""
        self.edges += ((n1, n2),)
        self.nodes = tuple(set(self.nodes) | {n1, n2})
        cdef int n
        for n in (n1, n2):
            self.adj[n] = self.neighbors(n)

    cpdef void add_path(self, object new_nodes):
        """Add path from a iterable."""
        cdef list edges = list(self.edges)
        cdef set nodes = set(self.nodes)

        cdef int n1 = -1
        cdef int n2
        for n2 in new_nodes:
            if n1 != -1:
                edges.append((n1, n2))
                nodes.update((n1, n2))
            n1 = n2

        self.edges = tuple(edges)
        self.nodes = tuple(nodes)

        for n1 in nodes:
            self.adj[n1] = self.neighbors(n1)

    cpdef void remove_edge(self, int n1, int n2):
        """Remove edge(s) {n1, n2} once if exist, otherwise do nothing."""
        cdef set nodes = set()
        cdef list edges = []

        cdef bint once = False
        cdef tuple edge
        for edge in self.edges:
            if {n1, n2} != set(edge) or once:
                nodes.update(edge)
                edges.append(edge)
            else:
                once = True

        self.edges = tuple(edges)
        self.nodes = tuple(nodes)

        cdef int n
        for n in self.nodes:
            edge = self.neighbors(n)
            if edge:
                self.adj[n] = edge
            else:
                self.adj.pop(n, None)

    cpdef int dof(self):
        """Return degrees of freedom."""
        return 3 * (len(self.nodes) - 1) - (2 * len(self.edges))

    cpdef inline tuple neighbors(self, int n):
        """Neighbors except the node."""
        cdef list neighbors = []
        cdef int l1, l2
        for l1, l2 in self.edges:
            if n == l1:
                neighbors.append(l2)
            if n == l2:
                neighbors.append(l1)
        return tuple(neighbors)

    cdef dict degrees(self):
        """Return number of neighbors par node."""
        return {n: len(neighbors) for n, neighbors in self.adj.items()}

    cpdef bint is_connected(self, int with_out = -1):
        """Return True if the graph is not isolated."""
        if not self.nodes:
            return True

        cdef int neighbors
        cdef int index = 0
        cdef list nodes = []
        # Change start node if index zero has been excluded.
        if with_out == self.nodes[0]:
            nodes.append(self.nodes[1])
        else:
            nodes.append(self.nodes[0])

        # Search node by node.
        while index < len(nodes):
            for neighbors in self.adj[nodes[index]]:
                if (neighbors not in nodes) and (neighbors != with_out):
                    nodes.append(neighbors)
            index += 1

        if with_out != -1:
            nodes.append(with_out)
        return len(nodes) == len(self.nodes)

    cpdef bint has_cut_link(self):
        """Return True if the graph has any cut links."""
        cdef int n, d
        for n, d in self.degrees().items():
            # Only for multiple links.
            if d > 2:
                # Remove a multiple link should be keep connected.
                if not self.is_connected(n):
                    return True
        return False

    cpdef bint is_degenerate(self):
        """Return True if this kinematic chain is degenerate.
        
        + Prue all multiple contracted links recursively.
        + Check the DOF of sub-graph if it is lower then zero.
        """
        if self.has_triangles():
            return True

        cdef int n1, n2
        cdef list c_l
        cdef Graph g = self.copy()
        cdef set mcl = set()
        while True:
            mcl.update(_multi_contracted_links(g, True))
            if not mcl:
                break

            c_l = []
            for n1, n2 in g.edges:
                if not {n1, n2} & mcl:
                    c_l.append((n1, n2))
            mcl.clear()

            # Pruned graph.
            g = Graph.__new__(Graph, c_l)
            if {n2 for n2 in g.degrees().values()} == {2}:
                # The graph is a basic loop.
                break

        # Check the DOF.
        return g.dof() < 1

    cpdef bint is_isomorphic(self, Graph g):
        """Return True if two graphs is isomorphic."""
        cdef GraphMatcher gm_gh = GraphMatcher.__new__(GraphMatcher, self, g)
        return gm_gh.is_isomorphic()

    cdef bint has_triangles(self):
        """Return True if the graph has triangles."""
        cdef int n1, n2
        cdef tuple neighbors
        for neighbors in self.adj.values():
            for n1 in neighbors:
                for n2 in neighbors:
                    if n1 == n2:
                        continue
                    if n1 in self.adj[n2]:
                        return True
        return False

    cpdef Graph copy(self):
        """Copy the graph."""
        return Graph.__new__(Graph, self.edges)

    def __repr__(self) -> str:
        """Print the edges."""
        return f"{self.__class__.__name__}({list(self.edges)})"


cdef bint _is_adjacent(Graph g, int u, int v):
    """Find the distance between u and v."""
    if v in g.adj[u]:
        return True
    return False


@cython.final
cdef class GraphMatcher:

    """GraphMatcher and GMState class from NetworkX.

    Copyright (C) 2007-2009 by the NetworkX maintainers
    All rights reserved.
    BSD license.

    This work was originally coded by Christopher Ellison
    as part of the Computational Mechanics Python (CMPy) project.
    James P. Crutchfield, principal investigator.
    Complexity Sciences Center and Physics Department, UC Davis.
    """

    cdef Graph g1, g2
    cdef set g1_nodes, g2_nodes
    cdef dict core_1, core_2, inout_1, inout_2, mapping
    cdef GMState state

    def __cinit__(self, g1: Graph, g2: Graph):
        self.g1 = g1
        self.g2 = g2
        self.g1_nodes = set(g1.nodes)
        self.g2_nodes = set(g2.nodes)

        # Set recursion limit.
        cdef int old_recursion_limit = sys.getrecursionlimit()
        cdef int expected_max_recursion_level = len(self.g2.nodes)
        if old_recursion_limit < 1.5 * expected_max_recursion_level:
            # Give some breathing room.
            sys.setrecursionlimit(int(1.5 * expected_max_recursion_level))

        # Initialize state
        self.initialize()

    cdef inline void initialize(self):
        """Re-initializes the state of the algorithm."""
        # core_1[n] contains the index of the node paired with n, which is m,
        #            provided n is in the mapping.
        # core_2[m] contains the index of the node paired with m, which is n,
        #            provided m is in the mapping.
        self.core_1 = {}
        self.core_2 = {}

        # See the paper for definitions of M_x and T_x^{y}
        # inout_1[n]  is non-zero if n is in M_1 or in T_1^{inout}
        # inout_2[m]  is non-zero if m is in M_2 or in T_2^{inout}
        #
        # The value stored is the depth of the SSR tree when the node became
        # part of the corresponding set.
        self.inout_1 = {}
        self.inout_2 = {}
        # Practically, these sets simply store the nodes in the subgraph.

        self.state = GMState.__new__(GMState, self)

        # Provide a convenient way to access the isomorphism mapping.
        self.mapping = self.core_1.copy()

    def candidate_pairs_iter(self) -> Iterator[Tuple[int, int]]:
        """Iterator over candidate pairs of nodes in g1 and g2."""
        # First we compute the inout-terminal sets.
        cdef set s1 = set(self.inout_1) - set(self.core_1)
        cdef set s2 = set(self.inout_2) - set(self.core_2)
        cdef list t1_inout = [node for node in self.g1_nodes if (node in s1)]
        cdef list t2_inout = [node for node in self.g2_nodes if (node in s2)]
        # If t1_inout and t2_inout are both nonempty.
        # P(s) = t1_inout x {min t2_inout}
        cdef int node
        if t1_inout and t2_inout:
            for node in t1_inout:
                yield node, min(t2_inout)
        else:
            # If t1_inout and t2_inout were both empty....
            # P(s) = (N_1 - M_1) x {min (N_2 - M_2)}
            # # if not (t1_inout or t2_inout):
            # as suggested by  [2], incorrect
            # as inferred from [1], correct
            # First we determine the candidate node for g2
            for node in self.g1.nodes:
                if node not in self.core_1:
                    yield node, min(self.g2_nodes - set(self.core_2))
        # For all other cases, we don't have any candidate pairs.

    cdef bint is_isomorphic(self):
        """Returns True if g1 and g2 are isomorphic graphs."""
        # Let's do two very quick checks!
        # QUESTION: Should we call faster_graph_could_be_isomorphic(g1,g2)?
        # For now, graph3 just copy the code.

        # Check global properties
        if len(self.g1.nodes) != len(self.g2.nodes):
            return False

        # Check local properties
        cdef tuple neighbor
        if (
            sorted([len(neighbor) for neighbor in self.g1.adj.values()])
            != sorted([len(neighbor) for neighbor in self.g2.adj.values()])
        ):
            return False

        self.initialize()
        try:
            next(self.match())
            return True
        except StopIteration:
            return False

    def match(self) -> Iterator[Dict[int, Tuple[int]]]:
        """Extends the isomorphism mapping."""
        cdef int g1_node, g2_node
        cdef GMState new_state
        if len(self.core_1) == len(self.g2.nodes):
            # Save the final mapping, otherwise garbage collection deletes it.
            self.mapping = self.core_1.copy()
            # The mapping is complete.
            yield self.mapping
        else:
            for g1_node, g2_node in self.candidate_pairs_iter():
                if self.syntactic_feasibility(g1_node, g2_node):
                    # Recursive call, adding the feasible state.
                    new_state = self.state.__class__(self, g1_node, g2_node)

                    yield from self.match()

                    # restore data structures
                    new_state.restore()

    cdef inline bint syntactic_feasibility(self, int g1_node, int g2_node):
        """Returns True if adding (g1_node, g2_node) is syntactically feasible.
        
        The VF2 algorithm was designed to work with graphs having, at most,
        one edge connecting any two nodes.  This is not the case when
        dealing with an MultiGraphs.
        
        Basically, when we test the look-ahead rules R_neighbor, we will
        make sure that the number of edges are checked. We also add
        a R_self check to verify that the number of self loops is acceptable.
        
        Users might be comparing Graph instances with MultiGraph instances.
        So the generic GraphMatcher class must work with MultiGraphs.
        Care must be taken since the value in the innermost dictionary is a
        singlet for Graph instances.  For MultiGraphs, the value in the
        innermost dictionary is a list.
        """

        # ## Test at each step to get a return value as soon as possible.

        # ## Look ahead 0
        # R_self
        # The number of self loops for g1_node must equal the number of
        # self-loops for g2_node. Without this check, we would fail on
        # R_neighbor at the next recursion level. But it is good to prune the
        # search tree now.
        if (
            _is_adjacent(self.g1, g1_node, g1_node)
            != _is_adjacent(self.g2, g2_node, g2_node)
        ):
            return False
        # R_neighbor
        # For each neighbor n' of n in the partial mapping, the corresponding
        # node m' is a neighbor of m, and vice versa. Also, the number of
        # edges must be equal.
        cdef int neighbor
        for neighbor in self.g1.adj[g1_node]:
            if neighbor in self.core_1:
                if self.core_1[neighbor] not in self.g2.adj[g2_node]:
                    return False
                elif (
                        _is_adjacent(self.g1, neighbor, g1_node)
                        != _is_adjacent(self.g2, self.core_1[neighbor], g2_node)
                ):
                    return False
        for neighbor in self.g2.adj[g2_node]:
            if neighbor in self.core_2:
                if self.core_2[neighbor] not in self.g1.adj[g1_node]:
                    return False
                elif (
                        _is_adjacent(self.g1, self.core_2[neighbor], g1_node)
                        != _is_adjacent(self.g2, neighbor, g2_node)
                ):
                    return False

        # ## Look ahead 1
        # R_term_inout
        # The number of neighbors of n that are in T_1^{inout} is equal to the
        # number of neighbors of m that are in T_2^{inout}, and vice versa.
        cdef int num1 = 0
        for neighbor in self.g1.adj[g1_node]:
            if (neighbor in self.inout_1) and (neighbor not in self.core_1):
                num1 += 1
        cdef int num2 = 0
        for neighbor in self.g2.adj[g2_node]:
            if (neighbor in self.inout_2) and (neighbor not in self.core_2):
                num2 += 1
        if num1 != num2:
            return False

        # ## Look ahead 2
        # R_new
        # The number of neighbors of n that are neither in the core_1 nor
        # T_1^{inout} is equal to the number of neighbors of m
        # that are neither in core_2 nor T_2^{inout}.
        num1 = 0
        for neighbor in self.g1.adj[g1_node]:
            if neighbor not in self.inout_1:
                num1 += 1
        num2 = 0
        for neighbor in self.g2.adj[g2_node]:
            if neighbor not in self.inout_2:
                num2 += 1
        return num1 == num2


@cython.final
cdef class GMState:

    cdef GraphMatcher gm
    cdef int g1_node, g2_node, depth

    def __cinit__(
        self,
        gm: GraphMatcher,
        g1_node: cython.int = -1,
        g2_node: cython.int = -1
    ):
        """Initializes GMState object.

        Pass in the GraphMatcher to which this GMState belongs and the
        new node pair that will be added to the GraphMatcher's current
        isomorphism mapping.
        """
        self.gm = gm

        # Initialize the last stored node pair.
        self.g1_node = -1
        self.g2_node = -1
        self.depth = len(gm.core_1)

        if g1_node == -1 or g2_node == -1:
            # Then we reset the class variables
            gm.core_1 = {}
            gm.core_2 = {}
            gm.inout_1 = {}
            gm.inout_2 = {}

        cdef int node
        cdef set new_nodes
        # Watch out! g1_node == 0 should evaluate to True.
        if g1_node != -1 and g2_node != -1:
            # Add the node pair to the isomorphism mapping.
            gm.core_1[g1_node] = g2_node
            gm.core_2[g2_node] = g1_node

            # Store the node that was added last.
            self.g1_node = g1_node
            self.g2_node = g2_node

            # Now we must update the other two vectors.
            # We will add only if it is not in there already!
            self.depth = len(gm.core_1)

            # First we add the new nodes...
            if g1_node not in gm.inout_1:
                gm.inout_1[g1_node] = self.depth
            if g2_node not in gm.inout_2:
                    gm.inout_2[g2_node] = self.depth

            # Now we add every other node...

            # Updates for T_1^{inout}
            new_nodes = set()
            for node in gm.core_1:
                new_nodes.update([
                    neighbor for neighbor in gm.g1.adj[node]
                    if neighbor not in gm.core_1
                ])
            for node in new_nodes:
                if node not in gm.inout_1:
                    gm.inout_1[node] = self.depth

            # Updates for T_2^{inout}
            new_nodes = set()
            for node in gm.core_2:
                new_nodes.update([
                    neighbor for neighbor in gm.g2.adj[node]
                    if neighbor not in gm.core_2
                ])
            for node in new_nodes:
                if node not in gm.inout_2:
                    gm.inout_2[node] = self.depth

    cdef void restore(self):
        """Deletes the GMState object and restores the class variables."""
        # First we remove the node that was added from the core vectors.
        # Watch out! g1_node == 0 should evaluate to True.
        if self.g1_node != -1 and self.g2_node != -1:
            del self.gm.core_1[self.g1_node]
            del self.gm.core_2[self.g2_node]

        # Now we revert the other two vectors.
        # Thus, we delete all entries which have this depth level.
        cdef dict vector
        cdef int node
        for vector in (self.gm.inout_1, self.gm.inout_2):
            for node in tuple(vector):
                if vector[node] == self.depth:
                    del vector[node]



# -*- coding: utf-8 -*-

from typing import (
    Tuple,
    List,
    Sequence,
    Dict,
    Optional,
    Union,
)
from .expression import VPoint

_Coord = Tuple[float, float]


def vpoint_solving(
    vpoints: Sequence[VPoint],
    inputs: Optional[Dict[Tuple[int, int], float]] = None,
    data_dict: Optional[Dict[Union[int, Tuple[int, int]], float]] = None
) -> List[Union[_Coord, Tuple[_Coord, _Coord]]]:
    """Solving function from vpoint list."""
    ...



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""Wrapper of BFGS algorithm.

Note of Pointer:
+ In Cython, pointer is more convenient then array.
    Because we can not "new" them or using "const" decorator on size_t.
+ There is NO pointer's "get value" operator in Cython,
    please use "index" operator.
+ Pointers can be plus with C's Integer, but not Python's.
    So please copy or declare to C's Integer.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

cimport cython
from cpython.mem cimport PyMem_Malloc, PyMem_Free
from libc.math cimport M_PI, cos, sin
from libcpp.list cimport list as clist
from libcpp.vector cimport vector
from libcpp.map cimport map as cmap
from libcpp.pair cimport pair
from pyslvs.sketch_solve cimport (
    Rough,
    Succsess,
    Point,
    Line,
    Constraint,
    PointOnPointConstraint,
    P2PDistanceConstraint,
    InternalAngleConstraint,
    PointOnLineConstraint,
    LineInternalAngleConstraint,
    solve,
)
from pyslvs.expression cimport (
    get_vlinks,
    VJoint,
    VPoint,
    VLink,
    Coordinate,
)


ctypedef fused T:
    double
    Line


cdef inline T *end_ptr(clist[T] *t_list):
    """Get last pointer."""
    return &t_list.back()


cdef inline void _sort_pairs(dict data_dict):
    """Sort the pairs in data_dict."""
    cdef object k
    for k in data_dict:
        if type(k) is tuple:
            data_dict[frozenset(k)] = data_dict.pop(k)


cdef inline double _radians(double degree):
    """Degrees to radians."""
    return degree / 180 * M_PI


cpdef list vpoint_solving(
    object vpoints_,
    dict inputs = None,
    dict data_dict = None
):
    """Solving function from vpoint list.

    + vpoints: Sequence[VPoint]
    + inputs: {(b0, d0): a0, (b1, d1): a1, ...}

    Known coordinates import from data_dict.
    + data_dict: {0: Coordinate(10.0, 20.0), ..., (0, 2): 30.0, ...}
    """
    # Sequences.
    cdef list vpoints = list(vpoints_)
    cdef dict vlinks = {vlink.name: vlink for vlink in get_vlinks(vpoints)}
    if inputs is None:
        inputs = {}
    if data_dict is None:
        data_dict = {}

    _sort_pairs(data_dict)

    cdef clist[double] params
    cdef clist[double] constants
    cdef vector[Point] points
    cdef cmap[int, int] sliders
    cdef vector[Point] slider_bases
    cdef vector[Point] slider_slots
    cdef clist[Line] slider_lines

    # Point parameters
    cdef int i
    cdef double x, y
    cdef double *tmp_ptr
    cdef VPoint vpoint
    cdef Coordinate coord
    for i, vpoint in enumerate(vpoints):
        if vpoint.no_link():
            x, y = vpoint.c[0]
            constants.push_back(x)
            tmp_ptr = end_ptr(&constants)
            constants.push_back(y)
            points.push_back([tmp_ptr, end_ptr(&constants)])
            continue

        if vpoint.grounded():
            if i in data_dict:
                # Known coordinates.
                coord = data_dict[i]
                constants.push_back(coord.x)
                tmp_ptr = end_ptr(&constants)
                constants.push_back(coord.y)
                points.push_back([tmp_ptr, end_ptr(&constants)])
                continue

            x, y = vpoint.c[0]
            constants.push_back(x)
            tmp_ptr = end_ptr(&constants)
            constants.push_back(y)
            if vpoint.type in {VJoint.P, VJoint.RP}:
                sliders[i] = <int>slider_bases.size()
                # Base point (slot) is fixed.
                slider_bases.push_back([tmp_ptr, end_ptr(&constants)])
                # Slot point (slot) is movable.
                params.push_back(x + cos(vpoint.angle))
                tmp_ptr = end_ptr(&params)
                params.push_back(y + sin(vpoint.angle))
                slider_slots.push_back([tmp_ptr, end_ptr(&params)])
                # Pin is movable.
                x, y = vpoint.c[1]
                if vpoint.has_offset() and vpoint.true_offset() <= 0.1:
                    if vpoint.offset() > 0:
                        x += 0.1
                        y += 0.1
                    else:
                        x -= 0.1
                        y -= 0.1
                params.push_back(x)
                tmp_ptr = end_ptr(&params)
                params.push_back(y)
                points.push_back([tmp_ptr, end_ptr(&params)])
            else:
                points.push_back([tmp_ptr, end_ptr(&constants)])
            continue

        if i in data_dict:
            # Known coordinates.
            coord = data_dict[i]
            constants.push_back(coord.x)
            tmp_ptr = end_ptr(&constants)
            constants.push_back(coord.y)
            points.push_back([tmp_ptr, end_ptr(&constants)])
            continue

        x, y = vpoint.c[0]
        params.push_back(x)
        tmp_ptr = end_ptr(&params)
        params.push_back(y)
        if vpoint.type in {VJoint.P, VJoint.RP}:
            sliders[i] = <int>slider_bases.size()
            # Base point (slot) is movable.
            slider_bases.push_back([tmp_ptr, end_ptr(&params)])
            # Slot point (slot) is movable.
            params.push_back(x + cos(vpoint.angle))
            tmp_ptr = end_ptr(&params)
            params.push_back(y + sin(vpoint.angle))
            slider_slots.push_back([tmp_ptr, end_ptr(&params)])
            if vpoint.pin_grounded():
                # Pin is fixed.
                x, y = vpoint.c[1]
                constants.push_back(x)
                tmp_ptr = end_ptr(&constants)
                constants.push_back(y)
                points.push_back([tmp_ptr, end_ptr(&constants)])
            else:
                # Pin is movable.
                x, y = vpoint.c[1]
                if vpoint.has_offset() and vpoint.true_offset() <= 0.1:
                    if vpoint.offset() > 0:
                        x += 0.1
                        y += 0.1
                    else:
                        x -= 0.1
                        y -= 0.1
                params.push_back(x)
                tmp_ptr = end_ptr(&params)
                params.push_back(y)
                points.push_back([tmp_ptr, end_ptr(&params)])
            continue

        # Point is movable.
        points.push_back([tmp_ptr, end_ptr(&params)])

    cdef clist[Constraint] cons_list

    # Link constraints
    cdef int a, b, c, d
    cdef frozenset frozen_pair
    cdef VPoint vp1, vp2
    cdef Point *p1
    cdef Point *p2
    cdef VLink vlink
    for vlink in vlinks.values():
        if len(vlink.points) < 2:
            continue
        if vlink.name == 'ground':
            continue

        a = vlink.points[0]
        b = vlink.points[1]
        if (a not in data_dict) or (b not in data_dict):
            vp1 = vpoints[a]
            vp2 = vpoints[b]
            if a not in data_dict and vp1.is_slot_link(vlink.name):
                p1 = &slider_bases[sliders[a]]
            else:
                p1 = &points[a]

            if b not in data_dict and vp2.is_slot_link(vlink.name):
                p2 = &slider_bases[sliders[b]]
            else:
                p2 = &points[b]

            frozen_pair = frozenset({a, b})
            if frozen_pair in data_dict:
                constants.push_back(data_dict[frozen_pair])
            else:
                constants.push_back(vp1.distance(vp2))

            cons_list.push_back(P2PDistanceConstraint(p1, p2, end_ptr(&constants)))

        for c in vlink.points[2:]:
            if c in data_dict:
                # Known coordinate.
                continue
            for d in (a, b):
                vp1 = vpoints[c]
                vp2 = vpoints[d]
                if vp1.is_slot_link(vlink.name):
                    p1 = &slider_bases[sliders[c]]
                else:
                    p1 = &points[c]

                if d not in data_dict and vp2.is_slot_link(vlink.name):
                    p2 = &slider_bases[sliders[d]]
                else:
                    p2 = &points[d]

                frozen_pair = frozenset({c, d})
                if frozen_pair in data_dict:
                    constants.push_back(data_dict[frozen_pair])
                else:
                    constants.push_back(vp1.distance(vp2))

                cons_list.push_back(P2PDistanceConstraint(p1, p2, end_ptr(&constants)))

    # Slider constraints
    cdef str name
    cdef Line *slider_slot
    cdef pair[int, int] slider
    for slider in sliders:
        a = slider.first
        b = slider.second
        # Base point
        vp1 = vpoints[a]
        p1 = &points[a]
        # Base slot
        slider_lines.push_back([&slider_bases[b], &slider_slots[b]])
        slider_slot = end_ptr(&slider_lines)
        if vp1.grounded():
            # Slot is grounded.
            constants.push_back(_radians(vp1.angle))
            cons_list.push_back(LineInternalAngleConstraint(slider_slot, end_ptr(&constants)))
            cons_list.push_back(PointOnLineConstraint(p1, slider_slot))
            if vp1.has_offset():
                p2 = &slider_bases[b]
                if vp1.offset():
                    constants.push_back(vp1.offset())
                    cons_list.push_back(P2PDistanceConstraint(p2, p1, end_ptr(&constants)))
                else:
                    cons_list.push_back(PointOnPointConstraint(p2, p1))
        else:
            # Slider between links.
            for name in vp1.links[:1]:
                vlink = vlinks[name]
                # A base link friend.
                c = vlink.points[0]
                if c == a:
                    if len(vlink.points) < 2:
                        # If no any friend.
                        continue
                    c = vlink.points[1]

                vp2 = vpoints[c]
                if vp2.is_slot_link(vlink.name):
                    # c is a slider, and it is be connected with slot link.
                    p2 = &slider_bases[sliders[c]]
                else:
                    # c is a R joint or it is not connected with slot link.
                    p2 = &points[c]
                slider_lines.push_back([&slider_bases[b], p2])
                constants.push_back(_radians(vp1.slope_angle(vp2) - vp1.angle))
                cons_list.push_back(InternalAngleConstraint(
                    slider_slot,
                    end_ptr(&slider_lines),
                    end_ptr(&constants)
                ))
                cons_list.push_back(PointOnLineConstraint(p1, slider_slot))

                if vp1.has_offset():
                    p2 = &slider_bases[b]
                    if vp1.offset():
                        constants.push_back(vp1.offset())
                        cons_list.push_back(P2PDistanceConstraint(p2, p1, end_ptr(&constants)))
                    else:
                        cons_list.push_back(PointOnPointConstraint(p2, p1))

        if vp1.type != VJoint.P:
            continue

        for name in vp1.links[1:]:
            vlink = vlinks[name]
            # A base link friend.
            c = vlink.points[0]
            if c == a:
                if len(vlink.points) < 2:
                    # If no any friend.
                    continue
                c = vlink.points[1]

            vp2 = vpoints[c]
            if vp2.is_slot_link(vlink.name):
                # c is a slider, and it is be connected with slot link.
                p2 = &slider_bases[sliders[c]]
            else:
                # c is a R joint or it is not connected with slot link.
                p2 = &points[c]
            slider_lines.push_back([p1, p2])
            constants.push_back(_radians(vp1.slope_angle(vp2) - vp1.angle))
            cons_list.push_back(InternalAngleConstraint(
                slider_slot,
                end_ptr(&slider_lines),
                end_ptr(&constants)
            ))

    # Angle constraints
    cdef clist[Line] handles
    cdef double angle
    for (b, d), angle in inputs.items():
        if b == d:
            continue
        handles.push_back([&points[b], &points[d]])
        constants.push_back(_radians(angle))
        cons_list.push_back(LineInternalAngleConstraint(
            end_ptr(&handles),
            end_ptr(&constants)
        ))

    # Pointer of parameters
    cdef int params_count = <int>params.size()
    cdef double **params_ptr = <double **>PyMem_Malloc(sizeof(double *) * params_count)
    cdef clist[double].iterator it = params.begin()
    for i in range(params_count):
        params_ptr[i] = &cython.operator.dereference(cython.operator.postincrement(it))

    # Pointer of constraints
    cdef int cons_count = <int>cons_list.size()
    cdef Constraint *cons = <Constraint *>PyMem_Malloc(sizeof(Constraint) * cons_count)
    i = 0
    cdef Constraint con
    for con in cons_list:
        cons[i] = con
        i += 1

    # Solve
    cdef int flag = solve(params_ptr, params_count, cons, cons_count, Rough)

    cdef list solved_points
    if flag == Succsess:
        solved_points = []
        for i, vpoint in enumerate(vpoints):
            if vpoint.type == VJoint.R:
                solved_points.append((points[i].x[0], points[i].y[0]))
            else:
                solved_points.append((
                    (slider_bases[sliders[i]].x[0], slider_bases[sliders[i]].y[0]),
                    (points[i].x[0], points[i].y[0])
                ))

    PyMem_Free(params_ptr)
    PyMem_Free(cons)

    if flag == Succsess:
        return solved_points
    else:
        raise ValueError("no valid solutions were found from initialed values")



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""The callable classes of the validation in algorithm.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

cimport cython
from numpy import (
    array as np_array,
    float as np_float,
    sort as np_sort,
)
# Not a number and a large fitness. Infinity cannot be used for a chart.
from libc.math cimport HUGE_VAL, NAN
from libcpp.list cimport list as clist
from numpy cimport ndarray
from pyslvs.Adesign.verify cimport Verification
from pyslvs.expression cimport (
    get_vlinks,
    VJoint,
    VPoint,
    VLink,
)
from pyslvs.triangulation cimport (
    vpoints_configure,
    symbol_str,
    Expression,
    PLA,
    PLLP,
    PLPP,
    PXY,
)
from pyslvs.bfgs cimport vpoint_solving
from pyslvs.tinycadlib cimport (
    radians,
    Coordinate,
    plap,
    pllp,
    plpp,
    pxy,
)


@cython.final
cdef class Planar(Verification):

    """This class is used to verified kinematics of the linkage mechanism."""

    cdef bint bfgs_mode
    cdef int target_count, base_index
    cdef clist[Expression] exprs
    cdef list vpoints, inputs, mapping_list
    cdef dict placement, target, mapping, mapping_r, data_dict
    cdef ndarray upper, lower

    def __cinit__(self, mech_params: dict):
        """mech_params = {
            'Expression': List[VPoint],
            'input': [(b0, d0), ...],
            'Placement': {pt: (x, y, r)},
            'Target': {pt: [(x0, y0), (x1, y1), ...]},
            'same': {pt: match_to_pt},
            # Bound has no position data.
            'upper': List[float],
            'lower': List[float],
        }
        """
        cdef dict placement = mech_params.get('Placement', {})
        if len(placement) == 0:
            raise ValueError("no grounded joint")

        cdef dict target = mech_params.get('Target', {})
        if len(target) == 0:
            raise ValueError("no target joint")

        cdef set check_set = set(map(len, target.values()))
        if len(check_set) != 1:
            raise ValueError("target paths should be in the same size")
        self.target_count = check_set.pop()

        cdef list inputs = list(mech_params.get('input', []))

        # Change the target paths into memory view.
        self.target = {}
        self.inputs = []
        cdef dict same = mech_params.get('same', {})

        cdef int i, j
        cdef Coordinate[:] path
        for i in target:
            path = np_array([Coordinate(x, y) for x, y in target[i]], dtype=object)
            for j in range(i):
                if j in same:
                    i -= 1
            self.target[i] = path

        cdef int a
        for i, j in inputs:
            for a in range(i):
                if a in same:
                    i -= 1
            for a in range(j):
                if a in same:
                    j -= 1
            self.inputs.append((i, j))

        # Options
        self.vpoints = list(mech_params.get('Expression', []))
        cdef dict status = {}
        self.exprs = vpoints_configure(self.vpoints, self.inputs, status).stack
        self.bfgs_mode = not all(status.values())

        # Bound
        cdef list upper = list(mech_params.get('upper', []))
        cdef list lower = list(mech_params.get('lower', []))
        if len(upper) != len(lower):
            raise ValueError("upper and lower should be in the same size")

        cdef list upper_input = []
        cdef list lower_input = []

        # Data mapping
        self.mapping = {i: f"P{i}" for i in range(len(self.vpoints))}
        self.mapping_r = {v: k for k, v in self.mapping.items()}
        self.mapping_list = []

        # Position
        cdef double x, y, r
        for i in sorted(placement):
            x, y, r = placement[i]
            upper_input.extend((x + r, y + r))
            lower_input.extend((x - r, y - r))
            self.mapping_list.append(i)

        # Length of links
        cdef int b, c, d
        cdef frozenset pair
        cdef VLink vlink
        for vlink in get_vlinks(self.vpoints):
            if len(vlink.points) < 2:
                continue
            a = vlink.points[0]
            b = vlink.points[1]
            pair = frozenset({a, b})
            self.mapping[pair] = None
            self.mapping_list.append(pair)
            for c in vlink.points[2:]:
                for d in (a, b):
                    pair = frozenset({c, d})
                    self.mapping[pair] = None
                    self.mapping_list.append(pair)

        upper_input.extend(upper[:-len(self.inputs)])
        lower_input.extend(lower[:-len(self.inputs)])

        for i in range(1, len(self.inputs) + 1):
            upper_input.extend([upper[-i]] * self.target_count)
            lower_input.extend([lower[-i]] * self.target_count)
        self.upper = np_array(upper_input, dtype=np_float)
        self.lower = np_array(lower_input, dtype=np_float)
        self.base_index = len(self.upper) - len(self.inputs) * self.target_count

        # Swap upper and lower bound if reversed.
        for i in range(len(self.upper)):
            if self.upper[i] < self.lower[i]:
                self.upper[i], self.lower[i] = self.lower[i], self.upper[i]

        # Result list
        self.data_dict = {}

    cdef ndarray[double, ndim=1] get_upper(self):
        return self.upper

    cdef ndarray[double, ndim=1] get_lower(self):
        return self.lower

    cpdef bint is_two_kernel(self):
        return self.bfgs_mode

    cdef inline double get_len(self, str expr1, str expr2):
        """Get the link length."""
        return self.mapping[frozenset({self.mapping_r[expr1], self.mapping_r[expr2]})]

    cdef inline bint solve(self, double[:] input_list):
        """Start solver function."""
        self.data_dict.clear()

        cdef int i
        cdef VPoint vpoint
        cdef Coordinate coord1, coord2
        for i, vpoint in enumerate(self.vpoints):
            if not vpoint.grounded():
                continue

            coord1 = Coordinate.__new__(Coordinate, vpoint.c[0][0], vpoint.c[0][1])
            if vpoint.type == VJoint.R:
                self.data_dict[self.mapping[i]] = coord1
                self.data_dict[i, -1] = coord1
            else:
                coord2 = Coordinate.__new__(Coordinate, vpoint.c[1][0], vpoint.c[1][1])
                self.data_dict[self.mapping[i]] = coord2
                self.data_dict[i, -1] = coord1
                self.data_dict[i, -2] = coord2

        # Solve
        i = 0
        cdef int t, params_count
        cdef Coordinate coord, coord3
        cdef str target
        cdef Expression expr
        for expr in self.exprs:
            coord = Coordinate.__new__(Coordinate, NAN, NAN)
            if expr.func == PLA:
                target = symbol_str(expr.c2)
                coord1 = self.data_dict[symbol_str(expr.c1)]
                coord = plap(
                    coord1,
                    self.get_len(symbol_str(expr.c1), target),
                    radians(input_list[i])
                )
                i += 1
            elif expr.func == PLLP:
                target = symbol_str(expr.c3)
                coord1 = self.data_dict[symbol_str(expr.c1)]
                coord2 = self.data_dict[symbol_str(expr.c2)]
                coord = pllp(
                    coord1,
                    self.get_len(symbol_str(expr.c1), target),
                    self.get_len(symbol_str(expr.c2), target),
                    coord2,
                    expr.op
                )
            elif expr.func == PLPP:
                target = symbol_str(expr.c4)
                coord1 = self.data_dict[symbol_str(expr.c1)]
                coord2 = self.data_dict[symbol_str(expr.c2)]
                coord3 = self.data_dict[symbol_str(expr.c3)]
                coord = plpp(
                    coord1,
                    self.get_len(symbol_str(expr.c1), target),
                    coord2,
                    coord3,
                    expr.op
                )
            elif expr.func == PXY:
                target = symbol_str(expr.c2)
                vpoint = self.vpoints[self.mapping_r[target]]
                coord1 = self.data_dict[symbol_str(expr.c1)]
                coord = pxy(coord1, vpoint.c[0][0] - coord1.x, vpoint.c[0][1] - coord1.y)
            else:
                return False

            if coord.is_nan():
                return False

            t = self.mapping_r[target]
            vpoint = self.vpoints[t]
            self.data_dict[target] = coord
            if vpoint.type == VJoint.R:
                self.data_dict[t, -1] = coord
            else:
                self.data_dict[t, -1] = vpoint.c[0]
                self.data_dict[t, -2] = coord

        if not self.bfgs_mode:
            return True

        # Calling Sketch Solve kernel and try to get the result.
        cdef dict p_data_dict
        # Add coordinate of known points.
        p_data_dict = {}
        for i in range(len(self.vpoints)):
            # {1: 'A'} vs {'A': (10., 20.)}
            if self.mapping[i] in self.data_dict:
                p_data_dict[i] = self.data_dict[self.mapping[i]]

        # Add specified link lengths.
        for k, v in self.mapping.items():
            if type(k) is frozenset:
                p_data_dict[k] = v

        # Solve
        cdef list solved_bfgs
        try:
            solved_bfgs = vpoint_solving(self.vpoints, {}, p_data_dict)
        except ValueError:
            return False

        # Format:
        # R joint: [[p0]: (p0_x, p0_y), [p1]: (p1_x, p1_y)]
        # P or RP joint: [[p2]: ((p2_x0, p2_y0), (p2_x1, p2_y1))]
        for i in range(len(self.vpoints)):
            if self.mapping[i] in self.data_dict:
                continue

            vpoint = self.vpoints[i]
            # These points solved by Sketch Solve.
            if vpoint.type == VJoint.R:
                self.data_dict[i, -1] = Coordinate.__new__(
                    Coordinate,
                    solved_bfgs[i][0],
                    solved_bfgs[i][1]
                )
            else:
                self.data_dict[i, -1] = Coordinate.__new__(
                    Coordinate,
                    solved_bfgs[i][0][0],
                    solved_bfgs[i][0][1]
                )
                self.data_dict[i, -2] = Coordinate.__new__(
                    Coordinate,
                    solved_bfgs[i][1][0],
                    solved_bfgs[i][1][1]
                )

        return True

    cdef double fitness(self, ndarray[double, ndim=1] v):
        """Chromosome format: (decided by upper and lower)

        v: [Ax, Ay, Dx, Dy, ..., L0, L1, ..., A00, A01, ..., A10, A11, ...]
        """
        cdef int target_index = 0
        cdef VPoint vpoint
        cdef object m
        for m in self.mapping_list:
            if type(m) is int:
                vpoint = self.vpoints[m]
                vpoint.locate(v[target_index], v[target_index + 1])
                target_index += 2
            else:
                self.mapping[m] = v[target_index]
                target_index += 1

        cdef double[:] input_list = np_sort(v[self.base_index:])
        cdef double fitness = 0.

        cdef int node
        cdef Coordinate coord
        cdef Coordinate[:] path
        for target_index in range(self.target_count):
            if not self.solve(input_list[target_index::self.target_count]):
                return HUGE_VAL

            for node, path in self.target.items():
                coord = self.data_dict[node, -1]
                fitness += coord.distance(path[target_index])

        return fitness

    cpdef object result(self, ndarray[double, ndim=1] v):
        """Return the last answer."""
        cdef int target_index = 0
        cdef VPoint vpoint
        cdef object m
        for m in self.mapping_list:
            if type(m) is int:
                vpoint = self.vpoints[m]
                vpoint.locate(v[target_index], v[target_index + 1])
                target_index += 2
            else:
                self.mapping[m] = v[target_index]
                target_index += 1

        cdef double[:] input_list = np_sort(v[self.base_index:])
        self.solve(input_list[::self.target_count])

        cdef list expressions = []

        cdef int i
        cdef double x1, y1, x2, y2
        cdef Coordinate coord1, coord2
        for i in range(len(self.vpoints)):
            vpoint = self.vpoints[i]
            coord1 = self.data_dict[i, -1]
            vpoint.locate(coord1.x, coord1.y)
            if vpoint.type != VJoint.R:
                coord1 = self.data_dict[i, -1]
                coord2 = self.data_dict[i, -2]
                vpoint.move((coord1.x, coord1.y), (coord2.x, coord2.y))
            expressions.append(vpoint.expr())

        return "M[" + ", ".join(expressions) + "]"



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""Structure synthesis.

The algorithms references:
+ On the Number Synthesis of Kinematic Chains
    + author: Hong-Sen Yan, Yu-Ting Chiu
    + Mechanism and Machine Theory, Volume 89, 2015, Pages 128-144, ISSN 0094-114X
    + https://doi.org/10.1016/j.mechmachtheory.2014.08.012

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

from time import time
from logging import getLogger
from collections import Counter
cimport cython
from libcpp.pair cimport pair as cpair
from libcpp.vector cimport vector
from libcpp.map cimport map as cmap
from numpy cimport int16_t
from numpy import (
    int16,
    array as np_array,
    zeros as np_zeros,
    ones as np_ones,
    sum as np_sum,
    subtract as np_sub,
    multiply as np_mul,
    floor_divide as np_div,
    any as np_any,
)
from pyslvs.number cimport product
from pyslvs.graph cimport Graph, link_assortment
from pyslvs.planar_check cimport is_planar

ctypedef unsigned int uint
ctypedef unsigned long long ullong
ctypedef cpair[int, int] ipair
ctypedef cmap[int, int] imap

cdef object logger = getLogger()


cdef inline int16_t[:] _nonzero_index(int16_t[:] array):
    """Return the number of nonzero numbers."""
    cdef list counter = []
    cdef int i, n
    for i, n in enumerate(array):
        if n != 0:
            counter.append(i)
    return np_array(counter, dtype=int16)


cdef inline ullong _factorial(int n):
    """Return (n!)."""
    cdef ullong ret = 1
    cdef int i
    for i in range(2, n + 1):
        ret *= i
    return ret


cdef inline int _gcd(int a, int b) nogil:
    """Return Greatest Common Divisor of a and b.
    
    Only for positive numbers.
    """
    cdef int r
    while b > 0:
        r = a % b
        a = b
        b = r
    return a


cdef inline int16_t[:] _labels(int16_t[:] numbers, int index, int offset):
    """Generate labels from numbers."""
    cdef int i, num
    cdef list labels = []
    for num in numbers[offset:]:
        for i in range(num):
            labels.append(index)
        index += 1
    return np_array(labels, dtype=int16)


cdef inline Graph _multigraph(int16_t[:] counter, int n):
    """Get multigraph from n x n matrix."""
    cdef dict edges = {}
    cdef int c = 0
    cdef int i, j
    for i in range(n):
        for j in range(n):
            if i >= j:
                continue
            edges[i, j] = counter[c]
            c += 1
    return Graph(Counter(edges).elements())


@cython.boundscheck(False)
cdef inline void _gauss_elimination(
    list result,
    int16_t[:] limit,
    int16_t[:, :] f_matrix,
    int n,
    int var_count
):
    """Gauss elimination for (n, n + 1) matrix."""
    cdef int i, j, d
    cdef int16_t[:] tmp1, tmp2
    for j in range(var_count):
        # Remove all coefficients of index [i] to zero.
        for i in range(n):
            if f_matrix[i, j] != 0 and not np_any(f_matrix[i, :j]):
                d = i
                tmp2 = np_div(f_matrix[i, :], f_matrix[i, j])
                break
        else:
            continue

        for i in range(n):
            if i == d or f_matrix[i, j] == 0:
                continue
            tmp1 = np_sub(f_matrix[i, :], np_mul(tmp2, f_matrix[i, j]))
            f_matrix[i, :] = tmp1

    # Answer
    cdef int16_t[:] answer = -np_ones(var_count, dtype=int16)

    # Determined solution
    cdef int c, k
    for i in range(n):
        c = 0
        for j in range(var_count):
            # Derivation (has answer)
            if answer[j] != -1 and f_matrix[i, j] != 0:
                f_matrix[i, -1] -= f_matrix[i, j] * answer[j]
                f_matrix[i, j] = 0

            # Nonzero coefficient
            if f_matrix[i, j] != 0:
                d = j
                c += 1

        if c != 1:
            continue

        j = d
        k = f_matrix[i, j]
        c = f_matrix[i, -1]
        if k != 1:
            if k < 0:
                k = -k
                c = -c
            if c < 0:
                return
            d = _gcd(k, c)
            k /= d
            c /= d
        if c < 0:
            return
        answer[j] = c

    # Result
    _test_contracted_graph(_multigraph(answer, n), result)


@cython.boundscheck(False)
cdef void _nest_do(
    list result,
    int16_t[:] answer,
    int16_t[:, :] f_matrix,
    int i,
    int n,
    object stop_func
):
    """Nest do loop."""
    if i >= n:
        # Result
        if <int>np_sum(answer) == <int>np_sum(f_matrix[:, -1]) / 2:
            _test_contracted_graph(_multigraph(answer, n), result)
        return

    cdef int16_t[:] coefficients = _nonzero_index(f_matrix[i, :-1])
    cdef int c = 0
    cdef int j = -1
    cdef int d
    for d in coefficients:
        if answer[d] == -1:
            c += 1
            j = d

    if c == 0:
        if <int>np_sum(np_mul(answer, f_matrix[i, :-1])) == f_matrix[i, -1]:
            _nest_do(result, answer, f_matrix, i + 1, n, stop_func)
        return
    elif c == 1:
        c = 0
        for d in coefficients:
            if answer[d] != -1:
                c += answer[d] * f_matrix[i, d]
        answer[j] = (f_matrix[i, -1] - c) / f_matrix[i, j]
        _nest_do(result, answer, f_matrix, i + 1, n, stop_func)
        return

    cdef int16_t[:] combine, answer_copy
    for combine in product((f_matrix[i, -1],) * c, stop_func):
        if stop_func is not None and stop_func():
            return

        c = 0
        d = 0
        for j in coefficients:
            if answer[j] == -1:
                c += combine[d] * f_matrix[i, j]
                d += 1
            else:
                c += answer[j] * f_matrix[i, j]
        if c != f_matrix[i, -1]:
            continue

        answer_copy = answer.copy()
        d = 0
        for j in coefficients:
            # Pass to answer
            if answer_copy[j] == -1:
                answer_copy[j] = combine[d]
                d += 1
        _nest_do(result, answer_copy, f_matrix, i + 1, n, stop_func)


cdef inline bint _is_isomorphic(Graph g, list result):
    """Return True if graph is isomorphic with result list."""
    cdef Graph h
    for h in result:
        if g.is_isomorphic(h):
            return True
    return False


cdef inline void _test_contracted_graph(Graph g, list result):
    """Test the contracted graph."""
    if not g.edges:
        return
    # All connected
    if not g.is_connected():
        return
    # Cut link
    if g.has_cut_link():
        return
    # Planar graph
    if not is_planar(g):
        return
    # Isomorphism
    if _is_isomorphic(g, result):
        return

    result.append(g)


cdef inline void _test_graph(
    Graph g,
    list result,
    uint no_degenerate
):
    """Test result graph."""
    # Graph filter depending on degenerate option
    if no_degenerate == 0 and not g.is_degenerate():
        return
    elif no_degenerate == 1 and g.is_degenerate():
        return
    # Isomorphism
    if _is_isomorphic(g, result):
        return
    result.append(g)


@cython.boundscheck(False)
cdef inline void _contracted_graph(
    list result,
    int16_t[:] limit,
    object stop_func
):
    """Synthesis of contracted graphs."""
    cdef int n = len(limit)
    cdef int var_count = n * (n - 1) / 2
    cdef int16_t[:, :] f_matrix = np_zeros((n, var_count + 1), dtype=int16)
    f_matrix[:, -1] = limit

    # Equations
    cdef int i, j, k, c
    for i in range(n):
        c = 0
        for j in range(n):
            for k in range(n):
                if j >= k:
                    continue
                if i in {j, k}:
                    f_matrix[i, c] = 1
                c += 1

    if n >= var_count:
        # Fast solution by Gauss elimination.
        _gauss_elimination(result, limit, f_matrix, n, var_count)
    else:
        # Nest do loop method.
        _nest_do(result, -np_ones(var_count, dtype=int16), f_matrix, 0, n, stop_func)


cdef inline void _dyad_insert(Graph g, frozenset edge, int amount):
    """Insert dyad to the graph."""
    if amount < 1:
        return

    cdef int last_num = max(g.nodes) + 1
    cdef int n1, n2
    n1, n2 = edge

    cdef list path = [n1]
    path.extend(range(last_num, last_num + amount))
    path.append(n2)

    g.remove_edge(n1, n2)
    g.add_path(path)


cdef inline void _permute_combine(
    int16_t[:] limit,
    list combine_list,
    tuple pick_list,
    object stop_func
):
    """Permutation of combined list."""
    cdef int n = len(limit)
    if n < 1:
        return

    cdef vector[int] indices = range(n)
    cdef int16_t[:] cycles = np_zeros(n, dtype=int16)
    cdef int16_t[:] pool = limit

    cdef set permute_list = set()

    cdef int i, j
    for i, j in enumerate(range(n, 0, -1)):
        cycles[i] = j

    permute_list.add(tuple(pool[indices[i]] for i in range(n)))

    cdef vector[int].iterator it2 = indices.begin()

    cdef int tmp
    while True:
        if stop_func is not None and stop_func():
            break

        for i in reversed(range(n)):
            cycles[i] -= 1
            if cycles[i] == 0:
                tmp = indices[i]
                indices.erase(it2 + i)
                indices.push_back(tmp)
                cycles[i] = n - i
            else:
                j = cycles[i]
                tmp = indices[i]
                indices[i] = indices[n - j]
                indices[n - j] = tmp
                permute_list.add(tuple(pool[indices[i]] for i in range(n)))
                break
        else:
            break

    cdef tuple tmp_array
    for tmp_array in permute_list:
        combine_list.append(tuple(zip(pick_list, tmp_array)))


cdef inline list _contracted_links(tuple edges, int16_t[:] limit, object stop_func):
    """Combination of contracted links.
    
    pool: edges
    pick: contracted links
    
    If the edge is not picked, it represent the joint is connected directly.
    """
    # Number of contracted links
    cdef int pick_count = len(limit)
    if pick_count < 1:
        return []

    # Check over picked
    cdef tuple pool_list = tuple([frozenset(edge) for edge in edges])
    cdef object pool = Counter(pool_list)
    # The list including required edge(s).
    cdef object confirm_list = pool - Counter(set(pool_list))
    # Simplified the pool
    pool_list = tuple((pool - confirm_list).elements())
    cdef int confirm_size = sum(confirm_list.values())
    if confirm_size > pick_count or pick_count > len(edges):
        return []

    pick_count -= confirm_size
    cdef int pool_size = len(pool_list)
    cdef int16_t[:] indices = np_zeros(pick_count, dtype=int16)
    cdef int i
    for i in range(pick_count):
        indices[i] = i

    cdef object pick_list = Counter()
    cdef set combine_set = set()

    # Combinations loop with number checking.
    cdef int n1, n2
    while True:
        if stop_func is not None and stop_func():
            break

        # Combine
        for i in range(pick_count):
            pick_list[pool_list[indices[i]]] += 1

        # Collecting
        combine_set.add(tuple(pick_list.elements()))

        # Initialize
        pick_list.clear()

        # Check combination is over.
        for n1 in reversed(range(pick_count)):
            if indices[n1] != n1 + pool_size - pick_count:
                break
        else:
            break

        # Next indicator
        indices[n1] += 1
        for n2 in range(n1 + 1, pick_count):
            indices[n2] = indices[n2 - 1] + 1

    cdef list combine_list = []
    pool_list = tuple(confirm_list.elements())
    cdef tuple picked
    for picked in combine_set:
        _permute_combine(limit, combine_list, pool_list + picked, stop_func)
    return combine_list


cdef inline void _graph_atlas(
    list result,
    list contracted_graph,
    int16_t[:] limit,
    uint no_degenerate,
    object stop_func
):
    """Synthesis of atlas."""
    cdef int n
    cdef Graph cg, g
    cdef tuple combine
    cdef frozenset edge
    for cg in contracted_graph:
        for combine in _contracted_links(cg.edges, limit, stop_func):
            g = Graph.__new__(Graph, cg.edges)
            for edge, n in combine:
                _dyad_insert(g, edge, n)
            _test_graph(g, result, no_degenerate)


cdef inline list _loop_chain(int num):
    """Loop chain of num."""
    cdef int i
    cdef int b = 0
    cdef list chain = []
    for i in range(1, num):
        chain.append((b, i))
        b = i
    chain.append((0, b))
    return chain


cpdef list contracted_graph(object link_num_list, object stop_func = None):
    """Get contracted graph by link assortment."""
    if not link_num_list:
        return []

    # Initial time
    cdef double t0 = time()
    cdef int16_t[:] link_num = np_array(link_num_list, ndmin=1, dtype=int16)
    logger.debug(f"Link assortment: {list(link_num)}")

    # Multiple links
    cdef int16_t[:] m_link = _labels(link_num, 3, 1)

    cdef imap m_limit, count
    cdef int num
    cdef int i = 0
    for num in m_link:
        m_limit[i] = num
        count[i] = 0
        i += 1

    # Synthesis of contracted graphs
    cdef list cg_list = []
    _contracted_graph(cg_list, m_link, stop_func)

    logger.debug(f"Contracted graph(s): {len(cg_list)}, time: {time() - t0}")
    return cg_list


cpdef list conventional_graph(
    list cg_list,
    object c_j_list,
    uint no_degenerate = 1,
    object stop_func = None
):
    """Linkage mechanism topological function.
    
    cg_list: Contracted graph list (List[Graph]).
    c_j_list = [NC1, NC2, NC3, ...]
    no_degenerate:
        0: only degenerate.
        1: no degenerate.
        2: all.
    stop_func: Optional[Callable[[], None]]
        stop function can check the break point and send response.
    """
    # Initial time
    cdef double t0 = time()
    logger.debug(f"Contracted link assortment: {list(c_j_list)}")

    # Synthesis of contracted link and multiple link combination.
    cdef int16_t[:] c_j = np_array(c_j_list, ndmin=1, dtype=int16)

    cdef list result = []
    cdef int i, num
    if not cg_list:
        if 1 not in c_j_list:
            return []
        # Single loop - ring graph (special case)
        i = 1
        for num in c_j:
            if num == 1:
                break
            i += 1
        else:
            raise ValueError("Invalid assortment!")
        result.append(Graph.__new__(Graph, _loop_chain(i)))
        logger.debug(f"Count: {len(result)}")
        return result

    # Multiple links
    cdef int16_t[:] m_link = np_array(link_assortment(cg_list[0]), ndmin=1, dtype=int16)
    m_link = _labels(m_link, 3, 1)

    # Synthesis of multiple links
    _graph_atlas(result, cg_list, _labels(c_j, 1, 0), no_degenerate, stop_func)

    # Return graph list and time
    logger.debug(f"Count: {len(result)}, time: {time() - t0}")
    return result



# -*- coding: utf-8 -*-
# cython: language_level=3, embedsignature=True, cdivision=True

"""Firefly Algorithm.

author: Yuan Chang
copyright: Copyright (C) 2016-2019
license: AGPL
email: pyslvs@gmail.com
"""

from time import time
cimport cython
from libc.math cimport (
    exp,
    log10,
    sqrt,
    HUGE_VAL,
)
from numpy cimport ndarray
from .verify cimport (
    Limit,
    MAX_GEN,
    MIN_FIT,
    MAX_TIME,
    rand_v,
    Chromosome,
    Verification,
)


cdef inline double _distance(Chromosome me, Chromosome she):
    """Distance of two fireflies."""
    cdef double dist = 0
    cdef int i
    cdef double diff
    for i in range(me.n):
        diff = me.v[i] - she.v[i]
        dist += diff * diff
    return sqrt(dist)


@cython.final
cdef class Firefly:

    """Algorithm class."""

    cdef Limit option
    cdef int D, n, max_gen, max_time, rpt, gen
    cdef double alpha, alpha0, beta_min, beta0, gamma, min_fit, time_start
    cdef Verification func
    cdef object progress_fun, interrupt_fun
    cdef ndarray lb, ub, fireflies
    cdef Chromosome last_best
    cdef list fitness_time

    def __cinit__(
        self,
        func: Verification,
        settings: dict,
        progress_fun: object = None,
        interrupt_fun: object = None
    ):
        """
        settings = {
            'n',
            'alpha',
            'beta_min',
            'beta0',
            'gamma',
            'max_gen', 'min_fit' or 'max_time',
            'report'
        }
        """
        # object function
        self.func = func

        # n, the population size of fireflies
        self.n = settings.get('n', 80)
        # alpha, the step size
        self.alpha = settings.get('alpha', 0.01)
        # alpha0, use to calculate_new_alpha
        self.alpha0 = self.alpha
        # beta_min, the minimal attraction, must not less than this
        self.beta_min = settings.get('beta_min', 0.2)
        # beta0, the attraction of two firefly in 0 distance.
        self.beta0 = settings.get('beta0', 1.)
        # gamma
        self.gamma = settings.get('gamma', 1.)

        # low bound
        self.lb = self.func.get_lower()
        # up bound
        self.ub = self.func.get_upper()
        if len(self.lb) != len(self.ub):
            raise ValueError("length of upper and lower bounds must be equal")
        # D, the dimension of question and each firefly will random place position in this landscape
        self.D = len(self.lb)

        # Algorithm will stop when the limitation has happened.
        self.max_gen = 0
        self.min_fit = 0
        self.max_time = 0
        if 'max_gen' in settings:
            self.option = MAX_GEN
            self.max_gen = settings['max_gen']
        elif 'min_fit' in settings:
            self.option = MIN_FIT
            self.min_fit = settings['min_fit']
        elif 'max_time' in settings:
            self.option = MAX_TIME
            self.max_time = settings['max_time']
        else:
            raise ValueError("please give 'max_gen', 'min_fit' or 'max_time' limit")
        # Report function
        self.rpt = settings.get('report', 0)
        if self.rpt <= 0:
            self.rpt = 10
        self.progress_fun = progress_fun
        self.interrupt_fun = interrupt_fun

        # all fireflies, depend on population n
        self.fireflies = ndarray(self.n, dtype=object)
        cdef int i
        for i in range(self.n):
            self.fireflies[i] = Chromosome.__new__(Chromosome, self.D)

        # generation of current
        self.gen = 0
        # best firefly so far
        self.last_best = Chromosome.__new__(Chromosome, self.D)

        # setup benchmark
        self.time_start = -1
        self.fitness_time = []

    cdef inline void initialize(self):
        cdef int i, j
        cdef Chromosome tmp
        for i in range(self.n):
            # initialize the Chromosome
            tmp = self.fireflies[i]
            for j in range(self.D):
                tmp.v[j] = rand_v(self.lb[j], self.ub[j])

    cdef inline void move_fireflies(self):
        cdef int i
        cdef bint is_move
        cdef double scale, tmp_v
        cdef Chromosome tmp, other
        for tmp in self.fireflies:
            is_move = False
            for other in self.fireflies:
                if tmp is other:
                    continue
                is_move |= self.move_firefly(tmp, other)
            if is_move:
                continue
            for i in range(self.D):
                scale = self.ub[i] - self.lb[i]
                tmp_v = tmp.v[i] + self.alpha * scale * rand_v(-0.5, 0.5)
                tmp.v[i] = self.check(i, tmp_v)

    cdef inline void evaluate(self):
        cdef Chromosome firefly
        for firefly in self.fireflies:
            firefly.f = self.func.fitness(firefly.v)

    cdef inline bint move_firefly(self, Chromosome me, Chromosome she):
        if me.f <= she.f:
            return False
        cdef double r = _distance(me, she)
        cdef double beta = (self.beta0 - self.beta_min) * exp(-self.gamma * r * r) + self.beta_min
        cdef int i
        cdef double scale, me_v
        for i in range(me.n):
            scale = self.ub[i] - self.lb[i]
            me_v = me.v[i] + beta * (she.v[i] - me.v[i]) + self.alpha * scale * rand_v(-0.5, 0.5)
            me.v[i] = self.check(i, me_v)
        return True

    cdef inline double check(self, int i, double v):
        if v > self.ub[i]:
            return self.ub[i]
        elif v < self.lb[i]:
            return self.lb[i]
        else:
            return v

    cdef inline Chromosome find_firefly(self):
        cdef int index = 0
        cdef double f = HUGE_VAL

        cdef int i
        cdef Chromosome tmp
        for i, tmp in enumerate(self.fireflies):
            tmp = self.fireflies[i]
            if tmp.f < f:
                index = i
                f = tmp.f
        return self.fireflies[index]

    cdef inline void report(self):
        self.fitness_time.append((self.gen, self.last_best.f, time() - self.time_start))

    cdef inline void generation_process(self):
        self.gen += 1

        self.move_fireflies()
        self.evaluate()
        # adjust alpha, depend on fitness value
        # if fitness value is larger, then alpha should larger
        # if fitness value is small, then alpha should smaller
        cdef Chromosome current_best = self.find_firefly()
        if self.last_best.f > current_best.f:
            self.last_best.assign(current_best)

        self.alpha = self.alpha0 * log10(current_best.f + 1)

        if self.gen % self.rpt == 0:
            self.report()

    cpdef tuple run(self):
        self.time_start = time()
        self.initialize()
        self.evaluate()
        self.last_best.assign(self.fireflies[0])
        self.report()

        while True:
            self.generation_process()

            if self.option == MAX_GEN:
                if self.gen >= self.max_gen > 0:
                    break
            elif self.option == MIN_FIT:
                if self.last_best.f <= self.min_fit:
                    break
            elif self.option == MAX_TIME:
                if time() - self.time_start >= self.max_time > 0:
                    break

            # progress
            if self.progress_fun is not None:
                self.progress_fun(self.gen, f"{self.last_best.f:.04f}")

            # interrupt
            if self.interrupt_fun is not None and self.interrupt_fun():
                break

        self.report()
        return self.func.result(self.last_best.v), self.fitness_time



# -*- coding: utf-8 -*-

from typing import (
    Tuple,
    List,
    Iterable,
)


def link_assortment(g: Graph) -> List[int]:
    """Return link assortment of the graph."""
    ...


def contracted_link_assortment(g: Graph) -> List[int]:
    """Return contracted link assortment of the graph."""
    ...


def labeled_enumerate(g: Graph) -> List[Tuple[int, Graph]]:
    """Enumerate each node with labeled except isomorphism."""
    ...


class Graph:

    """NetworkX-like graph class."""

    edges: Tuple[Tuple[int, int], ...]
    nodes: Tuple[int, ...]

    def __init__(self, edges: Iterable[Tuple[int, int]]): ...

    def add_edge(self, n1: int, n2: int):
        """Add two nodes for an edge."""
        ...

    def add_nodes(self, nodes: Iterable[int]):
        """Add nodes from a tuple."""
        ...

    def dof(self) -> int:
        """Return degrees of freedom."""
        ...

    def neighbors(self, n: int) -> Tuple[int, ...]:
        """Neighbors except the node."""
        ...

    def is_connected(self, with_out: int = ...) -> bool:
        """Return True if the graph is not isolated."""
        ...

    def has_cut_link(self) -> bool:
        """Return True if the graph has any cut links."""
        ...

    def is_degenerate(self) -> bool:
        """Return True if this kinematic chain is degenerate.

        + Prue all multiple contracted links recursively.
        + Check the DOF of sub-graph if it is lower then zero.
        """
        ...

    def is_isomorphic(self, graph: Graph) -> bool:
        """Return True if two graphs is isomorphic."""
        ...

    def copy(self) -> Graph:
        """Copy the graph."""
        ...



# -*- coding: utf-8 -*-

from typing import Any
import numpy as np


class Verification:

    """Verification function class base.

    The 'verify' module should be loaded when using sub-class.
    """

    def result(self, v: np.ndarray) -> Any:
        """Show the result."""
        ...



# -*- coding: utf-8 -*-

"""Compile the Cython libraries of Pyslvs."""

__author__ = "Yuan Chang"
__copyright__ = "Copyright (C) 2016-2019"
__license__ = "AGPL"
__email__ = "pyslvs@gmail.com"

import os
import re
import codecs
from setuptools import setup, Extension, find_packages
from setuptools.command.build_ext import build_ext
from platform import system

here = os.path.abspath(os.path.dirname(__file__))
src_path = 'pyslvs/'
bfgs_path = src_path + 'bfgs_solver/'
adesign_path = src_path + 'Adesign/'


def read(*parts):
    with codecs.open(os.path.join(here, *parts), 'r') as f:
        return f.read()


def find_version(*file_paths):
    m = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", read(*file_paths), re.M)
    if m:
        return m.group(1)
    raise RuntimeError("Unable to find version string.")


macros = [
    ('_hypot', 'hypot'),
]

compile_args = [
    '-O3',
    '-Wno-cpp',
]

if system() == 'Windows':
    # Avoid compile error with CYTHON_USE_PYLONG_INTERNALS.
    # https://github.com/cython/cython/issues/2670#issuecomment-432212671
    macros.append(('MS_WIN64', None))
    # Disable format warning.
    compile_args.append('-Wno-format')
    # Disable NumPy warning.
    macros.append(('NPY_NO_DEPRECATED_API', 'NPY_1_7_API_VERSION'))

ext_modules = [Extension(
    src_path.replace('/', '.') + 'bfgs',
    [
        src_path + 'bfgs.pyx',
        bfgs_path + 'geometric_constraint.cpp',
        bfgs_path + 'derivatives.cpp',
        bfgs_path + 'solve.cpp',
        bfgs_path + 'calc.cpp',
    ],
    language="c++",
    include_dirs=[bfgs_path],
    define_macros=macros,
    extra_compile_args=compile_args
)]

git_modules = [src_path, adesign_path]
for place in git_modules:
    for source in os.listdir(place):
        if not source.endswith('.pyx'):
            continue
        if place == src_path and source == 'bfgs.pyx':
            continue
        ext_modules.append(Extension(
            place.replace('/', '.') + source.split('.')[0],  # Base name
            [place + source],
            language="c++",
            include_dirs=[],
            define_macros=macros,
            extra_compile_args=compile_args
        ))


class Build(build_ext):
    def finalize_options(self):
        super(Build, self).finalize_options()
        # Prevent numpy from thinking it is still in its setup process
        if isinstance(__builtins__, dict):
            __builtins__['__NUMPY_SETUP__'] = False
        else:
            setattr(__builtins__, '__NUMPY_SETUP__', False)
        import numpy
        self.include_dirs.append(numpy.get_include())


setup(
    name='pyslvs',
    version=find_version('pyslvs', '__init__.py'),
    author=__author__,
    author_email=__email__,
    license=__license__,
    description="Pyslvs library",
    long_description=read("README.md"),
    long_description_content_type='text/markdown',
    url="https://github.com/KmolYuan/pyslvs",
    packages=find_packages(exclude=('tests',)),
    package_data={'': ["*.pyi"]},
    ext_modules=ext_modules,
    cmdclass={'build_ext': Build},
    zip_safe=False,
    python_requires=">=3.6",
    setup_requires=[
        'setuptools',
        'wheel',
        'numpy',
        'cython',
    ],
    install_requires=[
        'numpy',
        'lark-parser',
        'pygments',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Cython",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Operating System :: OS Independent",
    ]
)



# -*- coding: utf-8 -*-

from typing import (
    Tuple,
    List,
    Iterable,
    Callable,
    Optional,
)
from numpy import ndarray
from enum import IntEnum

# Color type must be a RGB data.
_Color = Tuple[int, int, int]


def get_vlinks(vpoints: Iterable[VPoint]) -> List[VLink]:
    """Get VLinks of a list of VPoint."""
    ...


class Coordinate:

    """A class to store the coordinate."""

    x: float
    y: float

    def __init__(self, x: float, y: float):
        ...

    def distance(self, p: Coordinate) -> float:
        """Distance."""
        ...

    def is_nan(self) -> bool:
        """Test this coordinate is a error-occurred answer."""
        ...

    def __repr__(self) -> str:
        ...


class VJoint(IntEnum):
    R: int
    P: int
    RP: int


class VPoint:

    """Symbol of joints."""

    links: Tuple[str, ...]
    c: ndarray
    type: int
    color: Optional[_Color]
    color_str: str
    type_str: str
    x: float
    y: float
    angle: float

    def __init__(
        self,
        links: Iterable[str],
        type_int: int,
        angle: float,
        color_str: str,
        x: float,
        y: float,
        color_func: Optional[Callable[..., _Color]] = None
    ):
        ...

    @staticmethod
    def r_joint(links: Iterable[str], x: float, y: float) -> VPoint:
        """Create by coordinate."""
        ...

    @staticmethod
    def slider_joint(links: Iterable[str], type_int: int, angle: float, x: float, y: float) -> VPoint:
        """Create by coordinate."""
        ...

    def copy(self) -> VPoint:
        """Copy method of Python."""
        ...

    @property
    def cx(self) -> float:
        """X value of first current coordinate."""
        return ...

    @property
    def cy(self) -> float:
        """Y value of first current coordinate."""
        return ...

    def move(self, c1: Tuple[float, float], c2: Optional[Tuple[float, float]] = None):
        """Change coordinates of this point."""
        ...

    def locate(self, x: float, y: float):
        """Change the origin coordinate of this point directly."""
        ...

    def rotate(self, angle: float):
        """Change the angle of slider slot by degrees."""
        ...

    def set_offset(self, offset: float):
        """Set slider offset."""
        ...

    def disable_offset(self):
        """Disable offset status."""
        ...

    def distance(self, p: VPoint) -> float:
        """Distance between two VPoint."""
        ...

    def has_offset(self) -> bool:
        """Return has offset."""
        ...

    def offset(self) -> float:
        """Return target offset."""
        ...

    def true_offset(self) -> float:
        """Return offset between slot and pin."""
        ...

    def slope_angle(self, p: VPoint, num1: int = 2, num2: int = 2) -> float:
        """Angle between horizontal line and two point.

        num1: me.
        num2: other side.
        [0]: base (slot) link.
        [1]: pin link.
        """
        ...

    def grounded(self) -> bool:
        """Return True if the joint is connect with the ground."""
        ...

    def pin_grounded(self) -> bool:
        """Return True if the joint has any pin connect with the ground."""
        ...

    def same_link(self, p: VPoint) -> bool:
        """Return True if the point is at the same link."""
        ...

    def no_link(self) -> bool:
        """Return True if the point has no link."""
        ...

    def is_slot_link(self, link_name: str) -> bool:
        """Return True if the link name is first link."""
        ...

    def expr(self) -> str:
        """Expression."""
        ...

    def __getitem__(self, i: int) -> float:
        ...

    def __repr__(self) -> str:
        ...


class VLink:

    """Symbol of links."""

    name: str
    color_str: str
    color: Optional[_Color]
    points: Tuple[int, ...]

    def __init__(
        self,
        name: str,
        color_str: str,
        points: Iterable[int],
        color_func: Optional[Callable[..., _Color]] = None
    ):
        ...

    def __contains__(self, point: int) -> bool:
        ...

    def __repr__(self) -> str:
        ...
